@@ -28,20 +28,40 @@
 /*                                                                 */
 /*******************************************************************/
 
+#if defined HAVE_PTHREAD_H && (defined HAVE_LIBPTHREAD || defined BUILDTIN_GCC_THREAD)
+extern pthread_mutex_t MUTEX_SYSCALL;
+extern pthread_mutex_t MUTEX_LOCK;
+#endif
+
+extern int PASS;
 extern RSA *PRIVKEY, *PUBKEY;
+extern char BINDINTERFACE[CF_BUFSIZE];
+extern char *ECGSOCKS[ATTR][2];
+extern char *TCPNAMES[CF_NETATTR];
+extern char **METHODARGV;
 
-/* cfengine */
-
+extern struct Item *METHODRETURNVARS;;
+extern struct Item *METHODRETURNCLASSES;
+extern char METHODFILENAME[CF_BUFSIZE];
+extern char *VMETHODPROTO[];
+extern int METHODARGC;
+extern char METHODREPLYTO[CF_BUFSIZE];
+extern char METHODFOR[CF_BUFSIZE];
+extern char METHODFORCE[CF_BUFSIZE];
 extern char CONTEXTID[32];
+extern char METHODNAME[CF_BUFSIZE];
+extern char METHODMD5[CF_BUFSIZE];
 extern char PADCHAR;
 extern struct cfagent_connection *CONN;
 extern int AUTHENTICATED;
+extern struct Item *IPADDRESSES;
 
-extern char CFLOCK[bufsize];
-extern char CFLOG[bufsize];
-extern char CFLAST[bufsize];
-extern char LOCKDB[bufsize];
-extern char EDITBUFF[bufsize];
+extern char CFLOCK[CF_BUFSIZE];
+extern char SAVELOCK[CF_BUFSIZE];
+extern char CFLOG[CF_BUFSIZE];
+extern char CFLAST[CF_BUFSIZE];
+extern char LOCKDB[CF_BUFSIZE];
+extern char EDITBUFF[CF_BUFSIZE];
 
 extern char *tzname[2];
 extern char *optarg;
@@ -52,11 +72,11 @@ extern char CFDES1[8];
 extern char CFDES2[8];
 extern char CFDES3[8];
 
-extern char CFPUBKEYFILE[bufsize];
-extern char CFPRIVKEYFILE[bufsize];
+extern char CFPUBKEYFILE[CF_BUFSIZE];
+extern char CFPRIVKEYFILE[CF_BUFSIZE];
 extern char AVDB[1024];
 
-
+extern dev_t ROOTDEVICE;
 extern char *VPRECONFIG;
 extern char *VRCFILE;
 
@@ -72,7 +92,7 @@ extern char *ACTIONTEXT[];
 extern char *ACTIONID[];
 extern char *BUILTINS[];
 extern char *CLASSTEXT[];
-extern char *CLASSATTRIBUTES[CLSSATTR][ATTRDIM];
+extern char *CLASSATTRIBUTES[CF_CLASSATTR][CF_ATTRDIM];
 extern char *FILEACTIONTEXT[];
 extern char *COMMATTRIBUTES[];
 extern char VINPUTFILE[];
@@ -112,6 +132,7 @@ extern int LINENUMBER;
 extern mode_t DEFAULTMODE;
 extern mode_t DEFAULTSYSTEMMODE;
 extern int HAVEUID;
+extern char *FINDERTYPE;
 extern char *VUIDNAME;
 extern char *VGIDNAME;
 extern char CFSERVER[];
@@ -133,6 +154,7 @@ extern char PRESERVETIMES;
 extern char TRUSTKEY;
 extern char FORK;
 
+extern short SKIPIDENTIFY;
 extern short COMPATIBILITY_MODE;
 extern short LINKSILENT;
 extern short UPDATEONLY;
@@ -146,15 +168,15 @@ extern char  INFORMP;
 
 extern char *FILTERNAME;
 extern char *STRATEGYNAME;
+extern char *CURRENTOBJECT;
 extern char *CURRENTITEM;
-extern char *CURRENTPATH;
 extern char *GROUPBUFF;
 extern char *ACTIONBUFF;
 extern char *CLASSBUFF;
-extern char ALLCLASSBUFFER[bufsize];
-extern char CHROOT[bufsize];
-extern char ELSECLASSBUFFER[bufsize];
-extern char FAILOVERBUFFER[bufsize];
+extern char ALLCLASSBUFFER[4*CF_BUFSIZE];
+extern char CHROOT[CF_BUFSIZE];
+extern char ELSECLASSBUFFER[CF_BUFSIZE];
+extern char FAILOVERBUFFER[CF_BUFSIZE];
 extern char *LINKFROM;
 extern char *LINKTO;
 extern char *ERROR;
@@ -169,6 +191,7 @@ extern char *CURRENTAUTHPATH;
 extern char *RESTART;
 extern char *FILTERDATA;
 extern char *STRATEGYDATA;
+extern char *PKGVER;
 
 extern short PROSIGNAL;
 extern char  PROACTION;
@@ -181,20 +204,20 @@ extern int IMGSIZE;
 extern char *CHECKSUMDB;
 extern char *COMPRESSCOMMAND;
 
-extern char *HASH[hashtablesize];
+extern char *HASH[CF_HASHTABLESIZE];
 
-extern char VBUFF[bufsize];
-extern char OUTPUT[bufsize*2];
+extern char VBUFF[CF_BUFSIZE];
+extern char OUTPUT[CF_BUFSIZE*2];
 
-extern char VFACULTY[maxvarsize];
-extern char VDOMAIN[maxvarsize];
-extern char VSYSADM[maxvarsize];
-extern char VNETMASK[maxvarsize];
-extern char VBROADCAST[maxvarsize];
-extern char VMAILSERVER[bufsize];
+extern char VFACULTY[CF_MAXVARSIZE];
+extern char VDOMAIN[CF_MAXVARSIZE];
+extern char VSYSADM[CF_MAXVARSIZE];
+extern char VNETMASK[CF_MAXVARSIZE];
+extern char VBROADCAST[CF_MAXVARSIZE];
+extern char VMAILSERVER[CF_BUFSIZE];
 extern struct Item *VTIMEZONE;
-extern char VDEFAULTROUTE[maxvarsize];
-extern char VNFSTYPE[maxvarsize];
+extern char VDEFAULTROUTE[CF_MAXVARSIZE];
+extern char VNFSTYPE[CF_MAXVARSIZE];
 extern char *VREPOSITORY;
 extern char *LOCALREPOS;
 extern char VIFNAME[16];
@@ -204,6 +227,8 @@ extern char VFQNAME[];
 extern char VUQNAME[];
 extern char LOGFILE[];
 
+extern short NOABSPATH;
+
 extern struct Item *VEXCLUDECACHE;
 extern struct Item *VSINGLECOPY;
 extern struct Item *VAUTODEFINE;
@@ -223,6 +248,10 @@ extern struct Item *VMOUNTLIST;
 extern struct Item *VHEAP;      /* Points to the base of the attribute heap */
 extern struct Item *VNEGHEAP;
 
+/* For packages: */
+extern struct Package *VPKG;
+extern struct Package *VPKGTOP;
+
 /* HvB : Bas van der Vlies */
 extern struct Mountables *VMOUNTABLES;  /* Points to the list of mountables */
 extern struct Mountables *VMOUNTABLESTOP;
@@ -230,6 +259,10 @@ extern struct Mountables *VMOUNTABLESTOP;
 extern struct cfObject *VOBJTOP;
 extern struct cfObject *VOBJ;
 
+
+extern char *PARSEMETHODRETURNCLASSES;
+
+extern struct Item *METHODARGS;
 extern flag  MOUNT_RO;                  /* mount directory readonly */
 
 extern struct Item *VALERTS;
@@ -291,9 +324,12 @@ extern struct UnMount *VUNMOUNTTOP;
 extern struct Item *VCLASSDEFINE;
 extern struct Image *VIMAGE;
 extern struct Image *VIMAGETOP;
+extern struct Method *VMETHODS;
+extern struct Method *VMETHODSTOP;
 extern struct Process *VPROCLIST;
 extern struct Process *VPROCTOP;
 extern struct Item *VSERVERLIST;
+extern struct Item *VRPCPEERLIST;
 extern struct Item *VREDEFINES;
 
 extern struct Item *VREPOSLIST;
@@ -316,6 +352,7 @@ extern short DEBUG;
 extern short D1;
 extern short D2;
 extern short D3;
+extern short D4;
 
 extern short PARSING;
 extern short ISCFENGINE;
@@ -325,6 +362,8 @@ extern short EXCLAIM;
 extern short INFORM;
 extern short CHECK;
 
+extern int PIFELAPSED;
+extern int PEXPIREAFTER;
 extern short LOGGING;
 extern short INFORM_save;
 extern short LOGGING_save;
@@ -332,7 +371,7 @@ extern short CFPARANOID;
 extern short SHOWACTIONS;
 extern short LOGTIDYHOMEFILES;
 
-extern short TIDYDIRS;
+extern char TIDYDIRS;
 extern short TRAVLINKS;
 extern short DEADLINKS;
 extern short PTRAVLINKS;
@@ -343,6 +382,7 @@ extern short GOTMOUNTINFO;
 extern short NOMOUNTS;
 extern short NOMODULES;
 extern short NOPROCS;
+extern short NOMETHODS;
 extern short NOFILECHECK;
 extern short NOTIDY;
 extern short NOSCRIPTS;
@@ -373,7 +413,9 @@ extern short UNDERSCORE_CLASSES;
 extern short NOHARDCLASSES;
 extern short NOSPLAY;
 extern short DONESPLAY;
+extern char XDEV;
 extern char TYPECHECK;
+extern char SCAN;
 
 extern enum actions ACTION;
 extern enum vnames CONTROLVAR;
@@ -412,6 +454,8 @@ extern char *VEDITNAMES[];
 extern char *VFILTERNAMES[];
 extern char *VUNMOUNTCOMM[];
 extern char *VRESOURCES[];
+extern char *CMPSENSETEXT[];
+extern char *PKGMGRTEXT[];
 
 extern int VTIMEOUT;
 extern mode_t UMASK;
@@ -432,6 +476,10 @@ extern int AUTOCREATED;
 
 extern enum fileactions FILEACTION;
 
+extern enum cmpsense CMPSENSE;
+extern enum pkgmgrs PKGMGR;
+extern enum pkgmgrs DEFAULTPKGMGR;
+
 extern unsigned short PORTNUMBER;
 
 extern int CURRENTLINENUMBER;
