@@ -51,22 +51,22 @@ int  MAXCHILD = 1;
 int  FileFlag = 0;
 int  TRUSTALL = false;
 enum fileoutputlevels  OUTPUTLEVEL = fopl_normal;
-char OUTPUTDIR[bufsize];
+char OUTPUTDIR[CF_BUFSIZE];
 
 struct Item *VCFRUNCLASSES = NULL;
 struct Item *VCFRUNOPTIONHOSTS = NULL;
 struct Item *VCFRUNHOSTLIST = NULL;
 
-char VCFRUNHOSTS[bufsize] = "cfrun.hosts";
-char CFRUNOPTIONS[bufsize];
-char CFLOCK[bufsize] = "dummy";
+char VCFRUNHOSTS[CF_BUFSIZE] = "cfrun.hosts";
+char CFRUNOPTIONS[CF_BUFSIZE];
+char CFLOCK[CF_BUFSIZE] = "dummy";
 
 /*******************************************************************/
 /* Functions internal to cfrun.c                                   */
 /*******************************************************************/
 
 void CheckOptsAndInit  ARGLIST((int argc,char **argv));
-int PollServer ARGLIST((char *host, char *options, int StoreInFile));
+int PollServer ARGLIST((char *host, char *options, int storeinfile));
 void SendClassData ARGLIST((int sd, char *sendbuffer));
 void CheckAccess ARGLIST((char *users));
 void cfrunSyntax ARGLIST((void));
@@ -78,15 +78,14 @@ void FileOutput ARGLIST((FILE *fp, enum fileoutputlevels level, char *message));
 /* Level 0 : Main                                                  */
 /*******************************************************************/
 
-int main (argc,argv)
-
-int argc;
-char **argv;
+int main (int argc,char **argv)
 
 { struct Item *ip;
-  int 	 i=0;
-  int	 status;
-  int	 pid;
+  int   i=0;
+  int  status;
+  int  pid;
+
+SetContext("cfrun");
 
 CheckOptsAndInit(argc,argv);
 
@@ -97,25 +96,25 @@ while (ip != NULL)
    if (i < MAXCHILD)
       {
       if (fork() == 0) /* child */
-	 {
-	 printf("cfrun(%d):         .......... [ Hailing %s ] ..........\n",i,ip->name);
-	 Debug("pid = %d i = %d\n", getpid(), i);
-	 
-	 if (PollServer(ip->name,ip->classes, FileFlag))
-	    {
-	    Verbose("Connection with %s completed\n\n",ip->name);
-	    }
-	 else
-	    {
-	    Verbose("Connection refused...\n\n");
-	    }
-	 exit(0);
-	 }
+         {
+         printf("cfrun(%d):         .......... [ Hailing %s ] ..........\n",i,ip->name);
+         Debug("pid = %d i = %d\n", getpid(), i);
+         
+         if (PollServer(ip->name,ip->classes, FileFlag))
+            {
+            Verbose("Connection with %s completed\n\n",ip->name);
+            }
+         else
+            {
+            Verbose("Connection refused...\n\n");
+            }
+         exit(0);
+         }
       else
-	 {
-	 /* parent */
-	 i++;
-	 }
+         {
+         /* parent */
+         i++;
+         }
       }
    else 
       {
@@ -123,6 +122,7 @@ while (ip != NULL)
       Debug("wait result pid = %d number %d\n", pid, i);
       i--;
       }
+
    if ( i < MAXCHILD )
       {
       ip = ip->next;
@@ -144,82 +144,79 @@ while (ip != NULL)
 /* Level 1                                                          */
 /********************************************************************/
 
-void CheckOptsAndInit(argc,argv)
-
-int argc;
-char **argv;
+void CheckOptsAndInit(int argc,char **argv)
 
 { int optgroup = 0, i;
   struct Item *ip;
 
 /* Separate command args into options and classes */
-bzero(CFRUNOPTIONS,bufsize);
+memset(CFRUNOPTIONS,0,CF_BUFSIZE);
 
 for (i = 1; i < argc; i++) 
    {
    if (optgroup == 0)
-      {	 
-      if (strncmp(argv[i],"-h",2) == 0)		 
-	 {
-	 cfrunSyntax();
-	 }
+      {  
+      if (strncmp(argv[i],"-h",2) == 0)   
+         {
+         cfrunSyntax();
+         }
       else if (strncmp(argv[i],"-f",2) == 0)
-	 {
-	 i++;
-	 
-	 if ((i >= argc) || (strncmp(argv[i],"-",1) == 0))
-	    {
-	    printf("Error: No filename listed after -f option.\n");
-	    cfrunSyntax();
-	    exit(0);
-	    }
-	 bzero(VCFRUNHOSTS,bufsize);
-	 strcat(VCFRUNHOSTS,argv[i]);
-	 Debug("cfrun: cfrun file = %s\n",VCFRUNHOSTS);
-	 }
+         {
+         i++;
+         
+         if ((i >= argc) || (strncmp(argv[i],"-",1) == 0))
+            {
+            printf("Error: No filename listed after -f option.\n");
+            cfrunSyntax();
+            exit(0);
+            }
+         memset(VCFRUNHOSTS,0,CF_BUFSIZE);
+         strncat(VCFRUNHOSTS,argv[i],CF_BUFSIZE-1-strlen(VCFRUNHOSTS));
+         Debug("cfrun: cfrun file = %s\n",VCFRUNHOSTS);
+         }
       else if (strncmp(argv[i],"-d",2) == 0)
-	 {
-	 DEBUG = true;
-	 VERBOSE = true;
-	 }
+         {
+         DEBUG = true;
+         VERBOSE = true;
+         }
       else if (strncmp(argv[i],"-v",2) == 0)
-	 {
-	 VERBOSE=true;
-	 }
+         {
+         VERBOSE=true;
+         }
       else if (strncmp(argv[i],"-T",2) == 0)
-	 {
-	 TRUSTALL = true;
-	 }
+         {
+         TRUSTALL = true;
+         }
       else if (strncmp(argv[i],"-S",2) == 0)
-	 {
-	 SILENT = true;
-	 }
+         {
+         SILENT = true;
+         }
       else if (strncmp(argv[i],"--",2) == 0) 
-	 {
-	 optgroup++;
-	 }
+         {
+         optgroup++;
+         }
       else if (argv[i][0] == '-')
-	 {
-	 printf("Error: Unknown option.\n");
-	 cfrunSyntax();
-	 exit(0);
-	 }
+         {
+         printf("Error: Unknown option.\n");
+         cfrunSyntax();
+         exit(0);
+         }
       else
-	 {
-	 AppendItem(&VCFRUNOPTIONHOSTS,argv[i],NULL); /* Restrict run hosts */
-	 }
+         {
+         AppendItem(&VCFRUNOPTIONHOSTS,argv[i],NULL); /* Restrict run hosts */
+         }
       }
    else if (optgroup == 1) 
       {
       if (strncmp(argv[i],"--",2) == 0)
-	 {
-	 optgroup++;
-	 }
+         {
+         optgroup++;
+         }
       else
-	 {
-	 strcat(CFRUNOPTIONS,argv[i]);
-	 strcat(CFRUNOPTIONS," ");
-	 }
+         {
+         strncat(CFRUNOPTIONS,argv[i],CF_BUFSIZE-1-strlen(CFRUNOPTIONS));
+         strncat(CFRUNOPTIONS," ",CF_BUFSIZE-1-strlen(CFRUNOPTIONS));
+         }
       }
    else
       {
@@ -227,17 +224,17 @@ for (i = 1; i < argc; i++)
       }
    }
  
-Debug("CFRUNOPTIONS string: %s\n",CFRUNOPTIONS);
-   
-for (ip = VCFRUNCLASSES; ip != NULL; ip=ip->next)
-   {
-   Debug("Class item: %s\n",ip->name);
-   }
-
-ReadCfrunConf(); 
-
-GetNameInfo();
+ Debug("CFRUNOPTIONS string: %s\n",CFRUNOPTIONS);
+ 
+ for (ip = VCFRUNCLASSES; ip != NULL; ip=ip->next)
+    {
+    Debug("Class item: %s\n",ip->name);
+    }
 
+ ReadCfrunConf(); 
+ 
+ GetNameInfo();
+ 
 /* 
 if (uname(&VSYSNAME) == -1)
    {
@@ -256,7 +253,7 @@ else
    }   
 */
  
- Debug("FQNAME = %s\n",VFQNAME);
+Debug("FQNAME = %s\n",VFQNAME);
  
 sprintf(VPREFIX,"cfrun:%s",VFQNAME);
  
@@ -271,25 +268,22 @@ ERR_load_crypto_strings();
 CheckWorkDirectories();
 LoadSecretKeys();
 RandomSeed();
-
 }
 
 
 /********************************************************************/
 
-int PollServer(host,options,StoreInFile)
-
-char *host, *options;
-int  StoreInFile;
+int PollServer(char *host,char *options,int storeinfile)
 
 { struct hostent *hp;
   struct sockaddr_in raddr;
-  char sendbuffer[bufsize],recvbuffer[bufsize];
-  char filebuffer[bufsize],parsed_host[bufsize];
+  char sendbuffer[CF_BUFSIZE],recvbuffer[CF_BUFSIZE];
+  char filebuffer[CF_BUFSIZE],parsed_host[CF_BUFSIZE];
   char reply[8];
   struct servent *server;
   int err,n_read,first,port;
   char *sp,forceipv4='n';
+  void *gotkey;
   FILE  *fp;
   struct Image addresses;
 #ifdef HAVE_GETADDRINFO
@@ -298,7 +292,7 @@ int  StoreInFile;
   
 CONN = NewAgentConn();
 
-if (StoreInFile)
+if (storeinfile)
    {
    sprintf(filebuffer, "%s/%s", OUTPUTDIR, host);
    if ((fp = fopen(filebuffer, "w")) == NULL)
@@ -319,20 +313,20 @@ FileVerbose(fp, "Connecting to server %s to port %d with options %s %s\n",parsed
 
  Debug("Using v6 compatible lookup...\n"); 
 
-bzero(&query,sizeof(struct addrinfo));
+memset(&query,0,sizeof(struct addrinfo));
 query.ai_family = AF_UNSPEC;
 query.ai_socktype = SOCK_STREAM;
 query.ai_flags = AI_PASSIVE;
  
 if ((err=getaddrinfo(parsed_host,NULL,&query,&response)) != 0)
    {
-   snprintf(OUTPUT,bufsize,"Unable to lookup %s (%s)",parsed_host,gai_strerror(err));
+   snprintf(OUTPUT,CF_BUFSIZE,"Unable to lookup %s (%s)",parsed_host,gai_strerror(err));
    CfLog(cferror,OUTPUT,"");
    }
  
 for (ap = response; ap != NULL; ap = ap->ai_next)
    {
-   snprintf(CONN->remoteip,cfmaxiplen,"%s",sockaddr_ntop(ap->ai_addr));
+   snprintf(CONN->remoteip,CF_MAX_IP_LEN,"%s",sockaddr_ntop(ap->ai_addr));
    break;
    }
 
@@ -353,7 +347,7 @@ if ((hp = gethostbyname(parsed_host)) == NULL)
    exit(1);
    }
  
-bzero(&raddr,sizeof(raddr));
+memset(&raddr,0,sizeof(raddr));
  
 if (port)
    {
@@ -376,18 +370,24 @@ else
 raddr.sin_addr.s_addr = ((struct in_addr *)(hp->h_addr))->s_addr;
 raddr.sin_family = AF_INET; 
 
-snprintf(CONN->remoteip,cfmaxiplen,"%s",inet_ntoa(raddr.sin_addr));
+snprintf(CONN->remoteip,CF_MAX_IP_LEN,"%s",inet_ntoa(raddr.sin_addr));
 
 #endif
  
 addresses.trustkey = 'n'; 
 addresses.encrypt = 'n';
 addresses.server = strdup(parsed_host);
- 
-snprintf(sendbuffer,bufsize,"root-%s",CONN->remoteip);
 
+snprintf(sendbuffer,CF_BUFSIZE,"root-%s",parsed_host);
+gotkey = HavePublicKey(sendbuffer);
  
-if (HavePublicKey(sendbuffer) == NULL)
+if (!gotkey)
+   {
+   snprintf(sendbuffer,CF_BUFSIZE,"root-%s",CONN->remoteip);
+   gotkey = HavePublicKey(sendbuffer);
+   }
+ 
+if (!gotkey)
    {
    if (TRUSTALL)
       {
@@ -401,24 +401,24 @@ if (HavePublicKey(sendbuffer) == NULL)
       printf("          Do you want to accept one on trust? (yes/no)\n\n--> ");
       
       while (true)
-	 {
-	 fgets(reply,8,stdin);
-	 Chop(reply);
-	 
-	 if (strcmp(reply,"yes")==0)
-	    {
-	    addresses.trustkey = 'y';
-	    break;
-	    }
-	 else if (strcmp(reply,"no")==0)
-	    {
-	    break;
-	    }
-	 else
-	    {
-	    printf("Please reply yes or no...(%s)\n",reply);
-	    }
-	 }
+         {
+         fgets(reply,8,stdin);
+         Chop(reply);
+         
+         if (strcmp(reply,"yes")==0)
+            {
+            addresses.trustkey = 'y';
+            break;
+            }
+         else if (strcmp(reply,"no")==0)
+            {
+            break;
+            }
+         else
+            {
+            printf("Please reply yes or no...(%s)\n",reply);
+            }
+         }
       }
    }
  
@@ -426,10 +426,10 @@ if (!RemoteConnect(parsed_host,forceipv4))
    {
    CfLog(cferror,"Couldn't open a socket","socket");
    FileOutput(fp, fopl_error, "Couldn't open a socket\n");
-   if (CONN->sd != cf_not_connected)
+   if (CONN->sd != CF_NOT_CONNECTED)
       {
       close(CONN->sd);
-      CONN->sd = cf_not_connected;
+      CONN->sd = CF_NOT_CONNECTED;
       }
    free(addresses.server);
    return false;
@@ -447,7 +447,7 @@ if (!IdentifyForVerification(CONN->sd,CONN->localip,CONN->family))
 
 if (!KeyAuthentication(&addresses))
    {
-   snprintf(OUTPUT,bufsize,"Key-authentication for %s failed\n",VFQNAME);
+   snprintf(OUTPUT,CF_BUFSIZE,"Key-authentication for %s failed\n",VFQNAME);
    CfLog(cferror,OUTPUT,"");
    FileOutput(fp, fopl_error, "Key-authentication failed\n");
    errno = EPERM;
@@ -456,7 +456,7 @@ if (!KeyAuthentication(&addresses))
    return false;
    }
  
-snprintf(sendbuffer,bufsize,"EXEC %s %s",options,CFRUNOPTIONS);
+snprintf(sendbuffer,CF_BUFSIZE,"EXEC %s %s",options,CFRUNOPTIONS);
 
 if (SendTransaction(CONN->sd,sendbuffer,0,CF_DONE) == -1)
    {
@@ -475,7 +475,7 @@ first = true;
  
 while (true)
    {
-   bzero(recvbuffer,bufsize);
+   memset(recvbuffer,0,CF_BUFSIZE);
 
    if ((n_read = ReceiveTransaction(CONN->sd,recvbuffer,NULL)) == -1)
       {
@@ -505,12 +505,12 @@ while (true)
    if (n_read == 0)
       {
       if (!first)
-	 {
-	 fprintf(fp,"\n");
-	 }
+         {
+         fprintf(fp,"\n");
+         }
       break;
       }
-
+   
    if (strlen(recvbuffer) == 0)
       {
       continue;
@@ -551,19 +551,18 @@ return true;
 /* Level 2                                                          */
 /********************************************************************/
 
-
 void ReadCfrunConf()
 
-{ char filename[bufsize], *sp;
-  char buffer[maxvarsize], options[bufsize], line[bufsize];
+{ char filename[CF_BUFSIZE], *sp;
+  char buffer[CF_MAXVARSIZE], options[CF_BUFSIZE], line[CF_BUFSIZE];
   FILE *fp;
   struct Item *ip; 
 
-bzero(filename,bufsize);
+memset(filename,0,CF_BUFSIZE);
 
 if (!strchr(VCFRUNHOSTS, '/'))
    {
-   if ((sp=getenv(CFINPUTSVAR)) != NULL)
+   if ((sp=getenv(CF_INPUTSVAR)) != NULL)
       {
       strcpy(filename,sp);
       if (filename[strlen(filename)-1] != '/')
@@ -573,7 +572,7 @@ if (!strchr(VCFRUNHOSTS, '/'))
       }
    else
       {
-      snprintf(filename, bufsize, "%s/inputs/", WORKDIR);
+      snprintf(filename, CF_BUFSIZE, "%s/inputs/", WORKDIR);
       }
    }
  
@@ -587,11 +586,11 @@ if ((fp = fopen(filename,"r")) == NULL)      /* Open root file */
 
 while (!feof(fp))
    {
-   bzero(buffer,maxvarsize);
-   bzero(options,bufsize);
-   bzero(line,bufsize);
+   memset(buffer,0,CF_MAXVARSIZE);
+   memset(options,0,CF_BUFSIZE);
+   memset(line,0,CF_BUFSIZE);
 
-   ReadLine(line,bufsize,fp);
+   ReadLine(line,CF_BUFSIZE,fp);
 
    if (strncmp(line,"domain",6) == 0)
       {
@@ -600,29 +599,43 @@ while (!feof(fp))
       continue;
       }
 
-   if (strncmp(line,"maxchild", strlen("maxchild")) == 0)
+   if (strncmp(line,"hostnamekeys",6) == 0)
       {
-      sscanf(line,"maxchild = %295[^# \n]", buffer);
+      char buf[16];
+      buf[0] = '\0';
+      sscanf(line,"hostnamekeys = %295[^# \n]",buf);
+      Verbose("Hostname keys\n");
 
-      if ( (MAXCHILD = atoi(buffer)) == 0 )
-	 {
-	 MAXCHILD = 1;
-	 }
-
-      Verbose("cfrun: maxchild = %d\n", MAXCHILD);
+      if (strcmp(buf,"yes") == 0||strcmp(buf,"true") == 0)
+         {
+         AddMacroValue(CONTEXTID,"HostnameKeys","true");
+         }
       continue;
       }
 
+   if (strncmp(line,"maxchild", strlen("maxchild")) == 0)
+      {
+      sscanf(line,"maxchild = %295[^# \n]", buffer);
+      
+      if ( (MAXCHILD = atoi(buffer)) == 0 )
+         {
+         MAXCHILD = 1;
+         }
+      
+      Verbose("cfrun: maxchild = %d\n", MAXCHILD);
+      continue;
+      }
+   
    if (strncmp(line,"outputdir", strlen("outputdir")) == 0)
       {
       sscanf(line,"outputdir = %295[^# \n]", OUTPUTDIR);
       Verbose("cfrun: outputdir = %s\n", OUTPUTDIR);
       if ( opendir(OUTPUTDIR) == NULL)
-	 {
+         {
          printf("Directory %s does not exists\n", OUTPUTDIR);
-	 exit(1);
+         exit(1);
          }
-
+      
       FileFlag=1;
       continue;
       }
@@ -630,47 +643,47 @@ while (!feof(fp))
    if (strncmp(line,"outputlevel", strlen("outputlevel")) == 0)
       {
       sscanf(line,"outputlevel = %295[^# \n]", buffer);
-
+      
       if (strncmp(buffer,"inform", strlen("inform")) == 0)
-	 {
-	 OUTPUTLEVEL = fopl_inform;
-	 Verbose("cfrun: outputlevel = inform");
-	 }
+         {
+         OUTPUTLEVEL = fopl_inform;
+         Verbose("cfrun: outputlevel = inform");
+         }
       else if (strncmp(buffer,"error", strlen("error")) == 0)
-	 {
-	 OUTPUTLEVEL = fopl_error;
-	 Verbose("cfrun: outputlevel = error");
-	 }
+         {
+         OUTPUTLEVEL = fopl_error;
+         Verbose("cfrun: outputlevel = error");
+         }
       else if (strncmp(buffer,"normal", strlen("normal")) == 0)
-	 {
-	 OUTPUTLEVEL = fopl_normal;
-	 Verbose("cfrun: outputlevel = normal");
-	 }
+         {
+         OUTPUTLEVEL = fopl_normal;
+         Verbose("cfrun: outputlevel = normal");
+         }
       else
          {
          printf("Invalid outputlevel: %s\n", OUTPUTDIR);
-	 }
-
+         }
+      
       continue;
       }
-
+   
    if (strncmp(line,"access",6) == 0)
       {
       for (sp = line; (*sp != '=') && (*sp != '\0'); sp++)
-	 {
-	 }
-
+         {
+         }
+      
       if (*sp == '\0' || *(++sp) == '\0')
-	 {
-	 continue;
-	 }
-
+         {
+         continue;
+         }
+      
       CheckAccess(sp);
       continue;
       }
-
+   
    sscanf(line,"%295s %[^#\n]",buffer,options);
-
+   
    if (buffer[0] == '#')
       {
       continue;
@@ -712,10 +725,7 @@ fclose(fp);
 
 /********************************************************************/
  
-int ParseHostname(hostname,new_hostname)
-
-char *hostname;
-char *new_hostname;
+int ParseHostname(char *hostname,char *new_hostname)
 
 { int port=0;
 
@@ -727,10 +737,7 @@ return(port);
 
 /********************************************************************/
 
-void SendClassData(sd, sendbuffer)
-
-int sd;
-char *sendbuffer;
+void SendClassData(int sd,char *sendbuffer)
 
 { struct Item *ip;
   int used;
@@ -738,11 +745,11 @@ char *sendbuffer;
 
 sp = sendbuffer;
 used = 0;
-bzero(sendbuffer,bufsize);
+memset(sendbuffer,0,CF_BUFSIZE);
   
 for (ip = VCFRUNCLASSES; ip != NULL; ip = ip->next)
    {
-   if (used + strlen(ip->name) +2 > bufsize)
+   if (used + strlen(ip->name) +2 > CF_BUFSIZE)
       {
       if (SendTransaction(sd,sendbuffer,0,CF_DONE) == -1)
          {
@@ -752,7 +759,7 @@ for (ip = VCFRUNCLASSES; ip != NULL; ip = ip->next)
 
       used = 0;
       sp = sendbuffer;
-      bzero(sendbuffer,bufsize);
+      memset(sendbuffer,0,CF_BUFSIZE);
       }
    
    strcat(sendbuffer,ip->name);
@@ -762,7 +769,7 @@ for (ip = VCFRUNCLASSES; ip != NULL; ip = ip->next)
    used += strlen(ip->name)+1;
    }
 
-if (used + strlen(CFD_TERMINATOR) +2 > bufsize)
+if (used + strlen(CFD_TERMINATOR) +2 > CF_BUFSIZE)
    {
    if (SendTransaction(sd,sendbuffer,0,CF_DONE) == -1)
       {
@@ -772,10 +779,10 @@ if (used + strlen(CFD_TERMINATOR) +2 > bufsize)
 
    used = 0;
    sp = sendbuffer;
-   bzero(sendbuffer,bufsize);
+   memset(sendbuffer,0,CF_BUFSIZE);
    }
    
-sprintf(sp,CFD_TERMINATOR);
+sprintf(sp, "%s", CFD_TERMINATOR);
 
 if (SendTransaction(sd,sendbuffer,0,CF_DONE) == -1)
    {
@@ -786,11 +793,9 @@ if (SendTransaction(sd,sendbuffer,0,CF_DONE) == -1)
 
 /********************************************************************/
 
-void CheckAccess(users)
+void CheckAccess(char *users)
 
-char *users;
-
-{ char id[maxvarsize], *sp;
+{ char id[CF_MAXVARSIZE], *sp;
   struct passwd *pw;
   int uid,myuid;
 
@@ -812,53 +817,49 @@ for (sp = users; *sp != '\0'; sp++)
    if (isalpha((int)id[0]))
       {
       if ((pw = getpwnam(id)) == NULL)
-	 {
-	 printf("cfrun: No such user (%s) in password database\n",id);
-	 continue;
-	 }
+         {
+         printf("cfrun: No such user (%s) in password database\n",id);
+         continue;
+         }
       
       if (pw->pw_uid == myuid)
-	 {
-	 return;
-	 }
+         {
+         return;
+         }
       }
    else
       {
       uid = atoi(id);
       if (uid == myuid)
-	 {
-	 return;
-	 }
+         {
+         return;
+         }
       }
    }
-
+ 
  printf("cfrun: you have not been granted permission to run cfrun\n");
  exit(0);
 }
 
 /********************************************************************/
 
-void FileOutput(fp, level, message)
-
-FILE *fp;
-enum fileoutputlevels level;
-char *message;
+void FileOutput(FILE *fp,enum fileoutputlevels level,char *message)
 
 {
  switch (level)
     {
     case fopl_inform:
-	if (OUTPUTLEVEL >= fopl_inform)
-	   {
-	   fprintf(fp, "cfrun: INFORM: %s", message);
-	   }
-	break;
+ if (OUTPUTLEVEL >= fopl_inform)
+    {
+    fprintf(fp, "cfrun: INFORM: %s", message);
+    }
+ break;
     case fopl_error:
-	if (OUTPUTLEVEL >= fopl_error)
-	   {
-	   fprintf(fp, "cfrun: ERROR: %s", message);
-	   }
-	break;
+ if (OUTPUTLEVEL >= fopl_error)
+    {
+    fprintf(fp, "cfrun: ERROR: %s", message);
+    }
+ break;
     /* Default is to do nothing. That's right for "normal"! */
     }
  return;
@@ -901,7 +902,7 @@ struct cfagent_connection *NewAgentConn()
 
 { struct cfagent_connection *ap = (struct cfagent_connection *)malloc(sizeof(struct cfagent_connection));
 
-ap->sd = cf_not_connected;
+ap->sd = CF_NOT_CONNECTED;
 ap->family = AF_INET; 
 ap->trust = false;
 ap->localip[0] = '\0';
@@ -930,41 +931,37 @@ ap = NULL;
 /* Linking simplification                                      */
 /***************************************************************/
 
-void FatalError(s)
-
-char *s;
+void FatalError(char *s)
 
 {
 exit(1);
 }
 
 
-int RecursiveTidySpecialArea(name,tp,maxrecurse,sb)
+int RecursiveTidySpecialArea(char *name,struct Tidy *tp,int maxrecurse,struct stat *sb)
 
-char *name;
-struct Tidy *tp;
-int maxrecurse;
-struct stat *sb;
 {
  return true;
 }
 
-int CompareMD5Net(file1,file2,ip)
-
-char *file1, *file2;
-struct Image *ip;
+int CompareMD5Net(char *file1,char *file2,struct Image *ip)
 
 {
  return 0;
 }
 
 
-void yyerror(s)
-
-char *s;
+void yyerror(char *s)
 
 {
 }
 
-/* EOF */
+void Warning(char *s)
+
+{
+if (WARNINGS)
+   {
+   fprintf (stderr,"%s:%s:%d: Warning: %s\n",VPREFIX,VCURRENTFILE,LINENUMBER,s);
+   }
+}
 
