@@ -4,8 +4,7 @@
         Free Software Foundation, Inc.
 
    This file is part of GNU cfengine - written and maintained 
-   by Mark Burgess, Dept of Computing and Engineering, Oslo College,
-   Dept. of Theoretical physics, University of Oslo
+   by Mark Burgess, Dept of Computing and Engineering, Oslo University College,
  
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -38,11 +37,10 @@
 /* Level 1                                                           */
 /*********************************************************************/
 
-int OpenServerConnection(ip)
+int OpenServerConnection(struct Image *ip)
 
-struct Image *ip;
-
-{
+{ char server[CF_EXPANDSIZE];
+ 
 if (strcmp(ip->server,"localhost") == 0)
    {
    AUTHENTICATED = true;
@@ -50,25 +48,28 @@ if (strcmp(ip->server,"localhost") == 0)
    }
 
 AUTHENTICATED = false; 
+ExpandVarstring(ip->server,server,NULL);
  
-if (CONN->sd == cf_not_connected)
-   {
-   Debug("Opening server connnection to %s\n",ip->server);
+if (CONN->sd == CF_NOT_CONNECTED)
+   {   
+   Debug("Opening server connnection to %s\n",server);
 
-   if (!RemoteConnect(ip->server,ip->forceipv4))
+   if (!RemoteConnect(server,ip->forceipv4))
       {
       CfLog(cfinform,"Couldn't open a socket","socket");
-      if (CONN->sd != cf_not_connected)
-	 {
-	 CloseServerConnection();
-	 }
+      if (CONN->sd != CF_NOT_CONNECTED)
+         {
+         CloseServerConnection();
+         }
       AUTHENTICATED = false;
       return false;
       }
+
+   Debug("Remote IP set to %s\n",CONN->remoteip);
    
    if (!IdentifyForVerification(CONN->sd,CONN->localip,CONN->family))
       {
-      snprintf(OUTPUT,bufsize,"Id-authentication for %s failed\n",VFQNAME);
+      snprintf(OUTPUT,CF_BUFSIZE,"Id-authentication for %s failed\n",VFQNAME);
       CfLog(cferror,OUTPUT,"");
       errno = EPERM;
       CloseServerConnection();
@@ -84,7 +85,7 @@ if (CONN->sd == cf_not_connected)
    
    else if (!KeyAuthentication(ip))
       {
-      snprintf(OUTPUT,bufsize,"Authentication dialogue with %s failed\n",ip->server);
+      snprintf(OUTPUT,CF_BUFSIZE,"Authentication dialogue with %s failed\n",server);
       CfLog(cferror,OUTPUT,"");
       errno = EPERM;
       CloseServerConnection();
@@ -97,7 +98,7 @@ if (CONN->sd == cf_not_connected)
    }
 else
    {
-   Debug("Server connection to %s already open on %d\n",ip->server,CONN->sd);
+   Debug("Server connection to %s already open on %d\n",server,CONN->sd);
    }
 
 AUTHENTICATED = true; 
@@ -113,7 +114,7 @@ Debug("Closing current connection\n");
 
 close(CONN->sd);
 
-CONN->sd = cf_not_connected;
+CONN->sd = CF_NOT_CONNECTED;
 
 if (CONN->session_key != NULL)
    {
@@ -124,27 +125,22 @@ if (CONN->session_key != NULL)
 
 /*********************************************************************/
 
-int cf_rstat(file,buf,ip,stattype)
+int cf_rstat(char *file,struct stat *buf,struct Image *ip,char *stattype)
 
 /* If a link, this reads readlink and sends it back in the same
    package. It then caches the value for each copy command */
 
-char *file;
-struct stat *buf;
-struct Image *ip;
-char *stattype;
-
-{ char sendbuffer[bufsize];
-  char recvbuffer[bufsize];
-  char in[bufsize],out[bufsize];
+{ char sendbuffer[CF_BUFSIZE];
+  char recvbuffer[CF_BUFSIZE];
+  char in[CF_BUFSIZE],out[CF_BUFSIZE];
   struct cfstat cfst;
   int ret,tosend,cipherlen;
   time_t tloc;
 
 Debug("cf_rstat(%s)\n",file);
-bzero(recvbuffer,bufsize); 
+memset(recvbuffer,0,CF_BUFSIZE); 
 
-if (strlen(file) > bufsize-30)
+if (strlen(file) > CF_BUFSIZE-30)
    {
    CfLog(cferror,"Filename too long","");
    return -1;
@@ -172,21 +168,21 @@ if (ip->encrypt == 'y')
       return -1;
       }
    
-   snprintf(in,bufsize-1,"SYNCH %d STAT %s",tloc,file);
+   snprintf(in,CF_BUFSIZE-1,"SYNCH %d STAT %s",tloc,file);
    cipherlen = EncryptString(in,out,CONN->session_key,strlen(in)+1);
-   snprintf(sendbuffer,bufsize-1,"SSYNCH %d",cipherlen);
-   bcopy(out,sendbuffer+CF_PROTO_OFFSET,cipherlen);
+   snprintf(sendbuffer,CF_BUFSIZE-1,"SSYNCH %d",cipherlen);
+   memcpy(sendbuffer+CF_PROTO_OFFSET,out,cipherlen);
    tosend = cipherlen+CF_PROTO_OFFSET;
    }
 else
    {
-   snprintf(sendbuffer,bufsize,"SYNCH %d STAT %s",tloc,file);
+   snprintf(sendbuffer,CF_BUFSIZE,"SYNCH %d STAT %s",tloc,file);
    tosend = strlen(sendbuffer);
    }
 
 if (SendTransaction(CONN->sd,sendbuffer,tosend,CF_DONE) == -1)
    {
-   snprintf(OUTPUT,bufsize*2,"Transmission failed/refused talking to %.255s:%.255s in stat",ip->server,file);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Transmission failed/refused talking to %.255s:%.255s in stat",ip->server,file);
    CfLog(cfinform,OUTPUT,"send");
    return -1;
    }
@@ -205,7 +201,7 @@ if (strstr(recvbuffer,"unsynchronized"))
 
 if (BadProtoReply(recvbuffer))
    {
-   snprintf(OUTPUT,bufsize*2,"Server returned error: %s\n",recvbuffer+4);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Server returned error: %s\n",recvbuffer+4);
    CfLog(cfverbose,OUTPUT,"");
    errno = EPERM;
    return -1;
@@ -213,10 +209,10 @@ if (BadProtoReply(recvbuffer))
 
 if (OKProtoReply(recvbuffer))
    {
-   long d1,d2,d3,d4,d5,d6,d7,d8,d9,d10,d11,d12=0;
+   long d1,d2,d3,d4,d5,d6,d7,d8,d9,d10,d11,d12=0,d13=0;
    
-   sscanf(recvbuffer,"OK: %1ld %5ld %14ld %14ld %14ld %14ld %14ld %14ld %14ld %14ld %14ld %14ld",
-	  &d1,&d2,&d3,&d4,&d5,&d6,&d7,&d8,&d9,&d10,&d11,&d12);
+   sscanf(recvbuffer,"OK: %1ld %5ld %14ld %14ld %14ld %14ld %14ld %14ld %14ld %14ld %14ld %14ld %14ld",
+   &d1,&d2,&d3,&d4,&d5,&d6,&d7,&d8,&d9,&d10,&d11,&d12,&d13);
 
    cfst.cf_type = (enum cf_filetype) d1;
    cfst.cf_mode = (mode_t) d2;
@@ -231,16 +227,17 @@ if (OKProtoReply(recvbuffer))
    ip->makeholes = (char) d10;
    cfst.cf_ino = d11;
    cfst.cf_nlink = d12;
+   cfst.cf_dev = d13;
 
    /* Use %?d here to avoid memory overflow attacks */
 
    Debug("Mode = %d,%d\n",d2,d3);
    
-   Debug("OK: type=%d\n mode=%o\n lmode=%o\n uid=%d\n gid=%d\n size=%ld\n atime=%d\n mtime=%d ino=%d nlnk=%d\n",
-	cfst.cf_type,cfst.cf_mode,cfst.cf_lmode,cfst.cf_uid,cfst.cf_gid,(long)cfst.cf_size,
-	cfst.cf_atime,cfst.cf_mtime,cfst.cf_ino,cfst.cf_nlink);
+   Debug("OK: type=%d\n mode=%o\n lmode=%o\n uid=%d\n gid=%d\n size=%ld\n atime=%d\n mtime=%d ino=%d nlnk=%d, dev=%d\n",
+ cfst.cf_type,cfst.cf_mode,cfst.cf_lmode,cfst.cf_uid,cfst.cf_gid,(long)cfst.cf_size,
+ cfst.cf_atime,cfst.cf_mtime,cfst.cf_ino,cfst.cf_nlink,cfst.cf_dev);
 
-   bzero(recvbuffer,bufsize);
+   memset(recvbuffer,0,CF_BUFSIZE);
    
    if (ReceiveTransaction(CONN->sd,recvbuffer,NULL) == -1)
       {
@@ -261,19 +258,19 @@ if (OKProtoReply(recvbuffer))
    switch (cfst.cf_type)
       {
       case cf_reg:   cfst.cf_mode |= (mode_t) S_IFREG;
-	             break;
+              break;
       case cf_dir:   cfst.cf_mode |= (mode_t) S_IFDIR;
-   	             break;
+                 break;
       case cf_char:  cfst.cf_mode |= (mode_t) S_IFCHR;
-	             break;
+              break;
       case cf_fifo:  cfst.cf_mode |= (mode_t) S_IFIFO;
-	             break;
+              break;
       case cf_sock:  cfst.cf_mode |= (mode_t) S_IFSOCK;
-	             break;
+              break;
       case cf_block: cfst.cf_mode |= (mode_t) S_IFBLK;
-	             break;
+              break;
       case cf_link:  cfst.cf_mode |= (mode_t) S_IFLNK;
-	             break;
+              break;
       }
 
 
@@ -310,13 +307,14 @@ if (OKProtoReply(recvbuffer))
    buf->st_ctime = cfst.cf_ctime;
    buf->st_atime = cfst.cf_atime;
    buf->st_ino   = cfst.cf_ino;
+   buf->st_dev   = cfst.cf_dev;
    buf->st_nlink = cfst.cf_nlink;
    
    return 0;
    }
 
 
-snprintf(OUTPUT,bufsize*2,"Transmission refused or failed statting %s\nGot: %s\n",file,recvbuffer); 
+snprintf(OUTPUT,CF_BUFSIZE*2,"Transmission refused or failed statting %s\nGot: %s\n",file,recvbuffer); 
 CfLog(cferror,OUTPUT,"");
 errno = EPERM;
 
@@ -325,20 +323,17 @@ return -1;
 
 /*********************************************************************/
 
-CFDIR *cf_ropendir(dirname,ip)
+CFDIR *cf_ropendir(char *dirname,struct Image *ip)
 
-char *dirname;
-struct Image *ip;
-
-{ char sendbuffer[bufsize];
-  char recvbuffer[bufsize];
+{ char sendbuffer[CF_BUFSIZE];
+  char recvbuffer[CF_BUFSIZE];
   int n, done=false;
   CFDIR *cfdirh;
   char *sp;
 
 Debug("CfOpenDir(%s:%s)\n",ip->server,dirname);
 
-if (strlen(dirname) > bufsize - 20)
+if (strlen(dirname) > CF_BUFSIZE - 20)
    {
    CfLog(cferror,"Directory name too long","");
    return NULL;
@@ -354,7 +349,7 @@ cfdirh->cf_list = NULL;
 cfdirh->cf_listpos = NULL;
 cfdirh->cf_dirh = NULL;
 
-snprintf(sendbuffer,bufsize,"OPENDIR %s",dirname);
+snprintf(sendbuffer,CF_BUFSIZE,"OPENDIR %s",dirname);
 
 if (SendTransaction(CONN->sd,sendbuffer,0,CF_DONE) == -1)
    {
@@ -379,14 +374,14 @@ while (!done)
 
    if (FailedProtoReply(recvbuffer))
       {
-      snprintf(OUTPUT,bufsize*2,"Network access to %s:%s denied\n",ip->server,dirname);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Network access to %s:%s denied\n",ip->server,dirname);
       CfLog(cfinform,OUTPUT,"");
       return false;      
       }
 
    if (BadProtoReply(recvbuffer))
       {
-      snprintf(OUTPUT,bufsize*2,"%s\n",recvbuffer+4);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"%s\n",recvbuffer+4);
       CfLog(cfinform,OUTPUT,"");
       return false;      
       }
@@ -394,29 +389,27 @@ while (!done)
    for (sp = recvbuffer; *sp != '\0'; sp++)
       {
       if (strncmp(sp,CFD_TERMINATOR,strlen(CFD_TERMINATOR)) == 0)        /* End transmission */
-	 {
-	 cfdirh->cf_listpos = cfdirh->cf_list;
-	 return cfdirh;
-	 }
-
+         {
+         cfdirh->cf_listpos = cfdirh->cf_list;
+         return cfdirh;
+         }
+      
       AppendItem(&(cfdirh->cf_list),sp,NULL);
       
       while(*sp != '\0')
-	 {
-	 sp++;
-	 }
+         {
+         sp++;
+         }
       }
    }
-
-cfdirh->cf_listpos = cfdirh->cf_list;
-return cfdirh;
+ 
+ cfdirh->cf_listpos = cfdirh->cf_list;
+ return cfdirh;
 }
 
 /*********************************************************************/
 
-void FlushClientCache(ip)
-
-struct Image *ip;
+void FlushClientCache(struct Image *ip)
 
 {
 if (ip->cache)
@@ -428,24 +421,21 @@ if (ip->cache)
 
 /*********************************************************************/
 
-int CompareMD5Net(file1,file2,ip)
-
-char *file1, *file2;
-struct Image *ip;
+int CompareMD5Net(char *file1,char *file2,struct Image *ip)
 
 { static unsigned char d[CF_MD5_LEN];
-  char *sp,sendbuffer[bufsize],recvbuffer[bufsize],in[bufsize],out[bufsize];
+  char *sp,sendbuffer[CF_BUFSIZE],recvbuffer[CF_BUFSIZE],in[CF_BUFSIZE],out[CF_BUFSIZE];
   int i,tosend,cipherlen;
 
 
 ChecksumFile(file2,d,'m');   /* send md5 to the server for comparison */
 Debug("Send digest of %s to server, %s\n",file2,ChecksumPrint('m',d));
 
-bzero(recvbuffer,bufsize);
+memset(recvbuffer,0,CF_BUFSIZE);
 
 if (ip->encrypt == 'y')
    {
-   snprintf(in,bufsize,"MD5 %s",file1);
+   snprintf(in,CF_BUFSIZE,"MD5 %s",file1);
 
    sp = in + strlen(in) + CF_SMALL_OFFSET;
 
@@ -455,13 +445,13 @@ if (ip->encrypt == 'y')
       }
    
    cipherlen = EncryptString(in,out,CONN->session_key,strlen(in)+CF_SMALL_OFFSET+CF_MD5_LEN);
-   snprintf(sendbuffer,bufsize,"SMD5 %d",cipherlen);
-   bcopy(out,sendbuffer+CF_PROTO_OFFSET,cipherlen);
+   snprintf(sendbuffer,CF_BUFSIZE,"SMD5 %d",cipherlen);
+   memcpy(sendbuffer+CF_PROTO_OFFSET,out,cipherlen);
    tosend = cipherlen + CF_PROTO_OFFSET;
    }
 else
    {
-   snprintf(sendbuffer,bufsize,"MD5 %s",file1);
+   snprintf(sendbuffer,CF_BUFSIZE,"MD5 %s",file1);
    sp = sendbuffer + strlen(sendbuffer) + CF_SMALL_OFFSET;
 
    for (i = 0; i < CF_MD5_LEN; i++)
@@ -500,24 +490,20 @@ else
 
 /*********************************************************************/
 
-int CopyRegNet(source,new,ip,size)
-
-char *source, *new;
-struct Image *ip;
-off_t size;
+int CopyRegNet(char *source,char *new,struct Image *ip,off_t size)
 
 { int dd, buf_size,n_read = 0,toget,towrite,plainlen,more = true;
   int last_write_made_hole = 0, done = false,tosend,cipherlen=0;
-  char *buf,in[bufsize],out[bufsize],sendbuffer[bufsize],cfchangedstr[265];
+  char *buf,in[CF_BUFSIZE],out[CF_BUFSIZE],sendbuffer[CF_BUFSIZE],cfchangedstr[265];
   unsigned char iv[] = {1,2,3,4,5,6,7,8};
   long n_read_total = 0;  
   EVP_CIPHER_CTX ctx;
 
-snprintf(cfchangedstr,255,"%s%s",CFCHANGEDSTR1,CFCHANGEDSTR2);
+snprintf(cfchangedstr,255,"%s%s",CF_CHANGEDSTR1,CF_CHANGEDSTR2);
   
 EVP_CIPHER_CTX_init(&ctx);  
 
-if ((strlen(new) > bufsize-20))
+if ((strlen(new) > CF_BUFSIZE-20))
    {
    CfLog(cferror,"Filename too long","");
    return false;
@@ -527,7 +513,7 @@ unlink(new);  /* To avoid link attacks */
   
 if ((dd = open(new,O_WRONLY|O_CREAT|O_TRUNC|O_EXCL|O_BINARY, 0600)) == -1)
    {
-   snprintf(OUTPUT,bufsize*2,"Copy %s:%s security - failed attempt to exploit a race? (Not copied)\n",ip->server,new);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Copy %s:%s security - failed attempt to exploit a race? (Not copied)\n",ip->server,new);
    CfLog(cferror,OUTPUT,"open");
    unlink(new);
    return false;
@@ -543,24 +529,17 @@ if (buf_size < 2048)
    } 
  
 if (ip->encrypt == 'y')
-   {
-   if (size < 17)
-      {
-      snprintf(OUTPUT,bufsize,"Cannot encrypt files smaller than 17 bytes with OpenSSL/Blowfish (%s)",source);
-      CfLog(cferror,OUTPUT,"");
-      return false;
-      }
-   
-   snprintf(in,bufsize-CF_PROTO_OFFSET,"GET dummykey %s",source);
+   {   
+   snprintf(in,CF_BUFSIZE-CF_PROTO_OFFSET,"GET dummykey %s",source);
    cipherlen = EncryptString(in,out,CONN->session_key,strlen(in)+1);
-   snprintf(sendbuffer,bufsize,"SGET %4d %4d",cipherlen,buf_size);
-   bcopy(out,sendbuffer+CF_PROTO_OFFSET,cipherlen);
+   snprintf(sendbuffer,CF_BUFSIZE,"SGET %4d %4d",cipherlen,buf_size);
+   memcpy(sendbuffer+CF_PROTO_OFFSET,out,cipherlen);
    tosend=cipherlen+CF_PROTO_OFFSET;   
    EVP_DecryptInit(&ctx,EVP_bf_cbc(),CONN->session_key,iv);
    }
 else
    {
-   snprintf(sendbuffer,bufsize,"GET %d %s",buf_size,source);
+   snprintf(sendbuffer,CF_BUFSIZE,"GET %d %s",buf_size,source);
    tosend=strlen(sendbuffer);
    }
 
@@ -571,7 +550,7 @@ if (SendTransaction(CONN->sd,sendbuffer,tosend,CF_DONE) == -1)
    return false;
    }
 
-buf = (char *) malloc(bufsize + sizeof(int)); /* Note bufsize not buf_size !! */
+buf = (char *) malloc(CF_BUFSIZE + sizeof(int)); /* Note CF_BUFSIZE not buf_size !! */
 n_read_total = 0;
 
 while (!done)
@@ -587,66 +566,69 @@ while (!done)
       towrite = (size - n_read_total);
       
       if (ip->compat == 'y')
-	 {
-	 toget = buf_size;
-	 }
+         {
+         toget = buf_size;
+         }
       else
-	 {
-	 toget = towrite;
-	 }
+         {
+         toget = towrite;
+         }
       }
    else
       {
       toget = towrite = 0;
       }
-
+   
    if (ip->encrypt == 'y')
       {
       if (more)
-	 {
-	 cipherlen = ReceiveTransaction(CONN->sd,buf,&more);
-	 }
+         {
+         if ((cipherlen = ReceiveTransaction(CONN->sd,buf,&more)) == -1)
+            {
+            return false;
+            }
+         }
       else
-	 {
-	 break;	 /* Already written last encrypted buffer */
-	 }
+         {
+         break;  /* Already written last encrypted buffer */
+         }
       }
    else
       {
       if ((n_read = RecvSocketStream(CONN->sd,buf,toget,0)) == -1)
-	 {
-	 if (errno == EINTR) 
-	    {
-	    continue;
-	    }
-	 
-	 CfLog(cferror,"Error in socket stream","recv");
-	 close(dd);
-	 free(buf);
-	 return false;
-	 }
+         {
+         if (errno == EINTR) 
+            {
+            continue;
+            }
+         
+         CfLog(cferror,"Error in socket stream","recv");
+         close(dd);
+         free(buf);
+         return false;
+         }
       }
-
-
+   
+   
    /* If the first thing we get is an error message, break. */
-
-   if (n_read_total == 0 && strncmp(buf,CFFAILEDSTR,strlen(CFFAILEDSTR)) == 0)
+   
+   if (n_read_total == 0 && strncmp(buf,CF_FAILEDSTR,strlen(CF_FAILEDSTR)) == 0)
       {
-      snprintf(OUTPUT,bufsize*2,"Network access to %s:%s denied\n",ip->server,source);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Network access to %s:%s denied\n",ip->server,source);
       if (ip->encrypt != 'y')
-	 {
-	 RecvSocketStream(CONN->sd,buf,buf_size-n_read,0); /* flush rest of transaction */
-	 }
+         {
+         RecvSocketStream(CONN->sd,buf,buf_size-n_read,0); /* flush rest of transaction */
+         }
       
       CfLog(cfinform,OUTPUT,"");
       close(dd);
       free(buf);
       return false;      
       }
-
+   
    if (strncmp(buf,cfchangedstr,strlen(cfchangedstr)) == 0)
       {
-      snprintf(OUTPUT,bufsize*2,"File %s:%s changed while copying\n",ip->server,source);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"File %s:%s changed while copying\n",ip->server,source);
       RecvSocketStream(CONN->sd,buf,buf_size-n_read,0); /* flush rest of transaction */
       CfLog(cfinform,OUTPUT,"");
       close(dd);
@@ -657,54 +639,57 @@ while (!done)
    if (ip->encrypt == 'y')
       {
       if (!EVP_DecryptUpdate(&ctx,sendbuffer,&plainlen,buf,cipherlen))
-	 {
-	 Debug("Decryption failed\n");
-	 return false;
-	 }
+         {
+         Debug("Decryption failed\n");
+         return false;
+         }
       
-      bcopy(sendbuffer,buf,plainlen);
+      memcpy(buf,sendbuffer,plainlen);
       n_read = towrite = plainlen;
       }
 
-   if (n_read == 0)
+   if (ip->encrypt != 'y')
       {
-      break;
+      if (n_read == 0)
+         {
+         break;
+         }
+      
+      if (n_read == size)
+         {
+         if (n_read_total == 0 && strncmp(buf,CF_FAILEDSTR,size) == 0)
+            {
+            snprintf(OUTPUT,CF_BUFSIZE*2,"Network access to %s:%s denied\n",ip->server,source);
+            CfLog(cfinform,OUTPUT,"");
+            close(dd);
+            free(buf);
+     return false;      
+            }
+         }
       }
-
-   if (n_read == size)
-      {
-      if (n_read_total == 0 && strncmp(buf,CFFAILEDSTR,size) == 0)
-	 {
-	 snprintf(OUTPUT,bufsize*2,"Network access to %s:%s denied\n",ip->server,source);
-	 CfLog(cfinform,OUTPUT,"");
-	 close(dd);
-	 free(buf);
-	 return false;      
-	 }
-      }
-
+   
 /*   if (n_read < toget)
-      {
-      snprintf(OUTPUT,bufsize*2,"Network error getting %s:%s\n",ip->server,source);
-      CfLog(cfinform,OUTPUT,"");
-      close(dd);
-      free(buf);
-      return false;  
-      }
+     {
+     snprintf(OUTPUT,CF_BUFSIZE*2,"Network error getting %s:%s\n",ip->server,source);
+     CfLog(cfinform,OUTPUT,"");
+     close(dd);
+     free(buf);
+     return false;  
+     }
 */   
    n_read_total += towrite; /* n_read; */
-
+   
    if (ip->encrypt == 'n')
       {
       if (n_read_total >= (long)size)  /* Handle EOF without closing socket */
-	 {
-	 done = true;
-	 }
+         {
+         done = true;
+         }
       }
-
+   
    if (!EmbeddedWrite(new,dd,buf,ip,towrite,&last_write_made_hole,n_read))
       {
-      snprintf(OUTPUT,bufsize,"Local disk write failed copying %s:%s to %s\n",ip->server,source,new);
+      snprintf(OUTPUT,CF_BUFSIZE,"Local disk write failed copying %s:%s to %s\n",ip->server,source,new);
       CfLog(cferror,OUTPUT,"");
       free(buf);
       unlink(new);
@@ -725,7 +710,7 @@ if (ip->encrypt == 'y') /* final crypto cleanup */
 
    if (!EmbeddedWrite(new,dd,buf,ip,plainlen,&last_write_made_hole,n_read))
       {
-      snprintf(OUTPUT,bufsize,"Local disk write failed copying %s:%s to %s\n",ip->server,source,new);
+      snprintf(OUTPUT,CF_BUFSIZE,"Local disk write failed copying %s:%s to %s\n",ip->server,source,new);
       CfLog(cferror,OUTPUT,"");
       free(buf);
       unlink(new);
@@ -766,12 +751,7 @@ return true;
 /* Level 2                                                           */
 /*********************************************************************/
 
-int GetCachedStatData(file,statbuf,ip,stattype)
-
-char *file;
-struct stat *statbuf;
-struct Image *ip;
-char *stattype;
+int GetCachedStatData(char *file,struct stat *statbuf,struct Image *ip,char *stattype)
 
 { struct cfstat *sp;
 
@@ -782,21 +762,21 @@ for (sp = ip->cache; sp != NULL; sp=sp->next)
    if ((strcmp(ip->server,sp->cf_server) == 0) && (strcmp(file,sp->cf_filename) == 0))
       {
       if (sp->cf_failed)  /* cached failure from cfopendir */
-	 {
-	 errno = EPERM;
-	 Debug("Cached failure to stat\n");
-	 return -1;
-	 }
-
+         {
+         errno = EPERM;
+         Debug("Cached failure to stat\n");
+         return -1;
+         }
+      
       if ((strcmp(stattype,"link") == 0) && (sp->cf_lmode != 0))
-	 {
-	 statbuf->st_mode  = sp->cf_lmode;
-	 }
+         {
+         statbuf->st_mode  = sp->cf_lmode;
+         }
       else
-	 {
+         {
          statbuf->st_mode  = sp->cf_mode;
-	 }
-
+         }
+      
       statbuf->st_uid   = sp->cf_uid;
       statbuf->st_gid   = sp->cf_gid;
       statbuf->st_size  = sp->cf_size;
@@ -805,22 +785,19 @@ for (sp = ip->cache; sp != NULL; sp=sp->next)
       statbuf->st_ctime = sp->cf_ctime;
       statbuf->st_ino   = sp->cf_ino;
       statbuf->st_nlink = sp->cf_nlink;      
-
+      
       Debug("Found in cache\n");
       return true;
       }
    }
-
-Debug("Did not find in cache\n"); 
-return false;
+ 
+ Debug("Did not find in cache\n"); 
+ return false;
 }
 
 /*********************************************************************/
 
-void CacheData(data,ip)
-
-struct cfstat *data;
-struct Image *ip;
+void CacheData(struct cfstat *data,struct Image *ip)
 
 { struct cfstat *sp;
 
@@ -830,7 +807,7 @@ if ((sp = (struct cfstat *) malloc(sizeof(struct cfstat))) == NULL)
    return;
    }
 
-bcopy(data,sp,sizeof(struct cfstat));
+memcpy(sp,data,sizeof(struct cfstat));
 
 sp->next = ip->cache;
 ip->cache = sp;
@@ -838,14 +815,12 @@ ip->cache = sp;
 
 /*********************************************************************/
 
-void FlushToEnd(sd,toget)
-
-int sd,toget;
+void FlushToEnd(int sd,int toget)
 
 { int i;
   char buffer[2]; 
 
-snprintf(OUTPUT,bufsize*2,"Flushing rest of file...%d bytes\n",toget);
+snprintf(OUTPUT,CF_BUFSIZE*2,"Flushing rest of file...%d bytes\n",toget);
 CfLog(cfinform,OUTPUT,""); 
  
 for (i = 0; i < toget; i++)
@@ -866,7 +841,7 @@ if ((ap = (struct cfagent_connection *)malloc(sizeof(struct cfagent_connection))
    }
 
 Debug("New server connection...\n");
-ap->sd = cf_not_connected;
+ap->sd = CF_NOT_CONNECTED;
 ap->family = AF_INET; 
 ap->trust = false;
 ap->localip[0] = '\0';
@@ -878,9 +853,7 @@ return ap;
 
 /*********************************************************************/
 
-void DeleteAgentConn(ap)
-
-struct cfagent_connection *ap;
+void DeleteAgentConn(struct cfagent_connection *ap)
 
 {
 if (ap->session_key != NULL)
