@@ -73,9 +73,7 @@ char *MONTHTEXT[] =
 /* Level 1                                                           */
 /*********************************************************************/
 
-int Day2Number(datestring)
-
-char *datestring;
+int Day2Number(char *datestring)
 
 { int i = 0;
 
@@ -92,11 +90,50 @@ return -1;
 
 /*********************************************************************/
 
-void AddInstallable(classlist)
+int CountParentheses(char *str)
 
-char *classlist;
+{ char *sp;
+  int count = 0;
 
-{ char *sp, currentitem[maxvarsize];
+for (sp = str; *sp != '\0'; sp++)
+   {
+   if (*sp == '(')
+      {
+      count++;
+      }
+
+   if (*sp == ')')
+      {
+      count--;
+      }
+   }
+
+return count;
+}
+
+/*********************************************************************/
+
+int NestedParentheses(char *str)
+
+{ char *sp;
+  int count = 0;
+
+for (sp = str; *sp != '\0'; sp++)
+   {
+   if (*sp == '(')
+      {
+      count++;
+      }
+   }
+
+return count;
+}
+
+/*********************************************************************/
+
+void AddInstallable(char *classlist)
+
+{ char *sp, currentitem[CF_MAXVARSIZE];
 
 if (classlist == NULL)
    {
@@ -127,29 +164,69 @@ Debug("AddInstallable(%s)\n",classlist);
 
 /*********************************************************************/
 
-void AddMultipleClasses(classlist)
+void AddPrefixedMultipleClasses(char *name,char *classlist)
 
-char *classlist;
-
-{ char *sp, currentitem[maxvarsize],local[maxvarsize];
+{ char *sp, currentitem[CF_MAXVARSIZE],local[CF_MAXVARSIZE],pref[CF_BUFSIZE];
  
 if ((classlist == NULL) || strlen(classlist) == 0)
    {
    return;
    }
 
-bzero(local,maxvarsize);
-strcpy(local,classlist);
+memset(local,0,CF_MAXVARSIZE);
+strncpy(local,classlist,CF_MAXVARSIZE-1);
+
+Debug("AddPrefixedMultipleClasses(%s,%s)\n",name,local);
+
+for (sp = local; *sp != '\0'; sp++)
+   {
+   memset(currentitem,0,CF_MAXVARSIZE);
+
+   sscanf(sp,"%250[^.:,]",currentitem);
+
+   sp += strlen(currentitem);
+
+   pref[0] = '\0';
+   snprintf(pref,CF_BUFSIZE,"%s_%s",name,currentitem);
+
+   
+   if (IsHardClass(pref))
+      {
+      FatalError("cfengine: You cannot use -D to define a reserved class!");
+      }
+
+   AddClassToHeap(CanonifyName(pref));
+   }
+}
+
+/*********************************************************************/
+
+void AddMultipleClasses(char *classlist)
+
+{ char *sp, currentitem[CF_MAXVARSIZE],local[CF_MAXVARSIZE];
+ 
+if ((classlist == NULL) || strlen(classlist) == 0)
+   {
+   return;
+   }
+
+memset(local,0,CF_MAXVARSIZE);
+strncpy(local,classlist,CF_MAXVARSIZE-1);
 
 Debug("AddMultipleClasses(%s)\n",local);
 
 for (sp = local; *sp != '\0'; sp++)
    {
-   bzero(currentitem,maxvarsize);
+   memset(currentitem,0,CF_MAXVARSIZE);
 
    sscanf(sp,"%250[^.:,]",currentitem);
 
    sp += strlen(currentitem);
+      
+   if (IsHardClass(currentitem))
+      {
+      FatalError("cfengine: You cannot use -D to define a reserved class!");
+      }
 
    AddClassToHeap(CanonifyName(currentitem));
    }
@@ -157,9 +234,7 @@ for (sp = local; *sp != '\0'; sp++)
 
 /*********************************************************************/
 
-void AddTimeClass(str)
-
-char *str;
+void AddTimeClass(char *str)
 
 { int i;
   char buf2[10], buf3[10], buf4[10], buf5[10], buf[10], out[10];
@@ -180,7 +255,7 @@ sscanf(str,"%*s %s %s %s %s",buf2,buf3,buf4,buf5);
 sscanf(buf4,"%[^:]",buf);
 sprintf(out,"Hr%s",buf);
 AddClassToHeap(out);
-bzero(VHR,3);
+memset(VHR,0,3);
 strncpy(VHR,buf,2); 
 
 /* Minutes */
@@ -188,7 +263,7 @@ strncpy(VHR,buf,2);
 sscanf(buf4,"%*[^:]:%[^:]",buf);
 sprintf(out,"Min%s",buf);
 AddClassToHeap(out);
-bzero(VMINUTE,3);
+memset(VMINUTE,0,3);
 strncpy(VMINUTE,buf,2); 
  
 sscanf(buf,"%d",&i);
@@ -227,19 +302,19 @@ switch ((i / 15))
    {
    case 0: AddClassToHeap("Q1");
            sprintf(out,"Hr%s_Q1",VHR);
-	   AddClassToHeap(out);
+    AddClassToHeap(out);
            break;
    case 1: AddClassToHeap("Q2");
            sprintf(out,"Hr%s_Q2",VHR);
-	   AddClassToHeap(out);
+    AddClassToHeap(out);
            break;
    case 2: AddClassToHeap("Q3");
            sprintf(out,"Hr%s_Q3",VHR);
-	   AddClassToHeap(out);
+    AddClassToHeap(out);
            break;
    case 3: AddClassToHeap("Q4");
            sprintf(out,"Hr%s_Q4",VHR);
-	   AddClassToHeap(out);
+    AddClassToHeap(out);
            break;
    }
  
@@ -248,7 +323,7 @@ switch ((i / 15))
 
 sprintf(out,"Day%s",buf3);
 AddClassToHeap(out);
-bzero(VDAY,3);
+memset(VDAY,0,3);
 strncpy(VDAY,buf3,2);
  
 /* Month */
@@ -258,7 +333,7 @@ for (i = 0; i < 12; i++)
    if (strncmp(MONTHTEXT[i],buf2,3)==0)
       {
       AddClassToHeap(MONTHTEXT[i]);
-      bzero(VMONTH,4);
+      memset(VMONTH,0,4);
       strncpy(VMONTH,MONTHTEXT[i],3);
       break;
       }
@@ -274,9 +349,7 @@ AddClassToHeap(out);
 
 /*******************************************************************/
 
-int Month2Number(string)
-
-char *string;
+int Month2Number(char *string)
 
 { int i;
 
@@ -299,14 +372,20 @@ return -1;
 
 /*******************************************************************/
 
-void AddClassToHeap(class)
-
-char *class;
+void AddClassToHeap(char *class)
 
 {
 Debug("AddClassToHeap(%s)\n",class);
 
- if (IsItemIn(VHEAP,class))
+/*if (! IsItemIn(VALLADDCLASSES,class))
+   {
+   snprintf(OUTPUT,CF_BUFSIZE,"Defining class %s -- but it isn't declared installable",class);
+   yyerror(OUTPUT);
+   }*/
+
+Chop(class);
+ 
+if (IsItemIn(VHEAP,class))
    {
    return;
    }
@@ -316,9 +395,7 @@ AppendItem(&VHEAP,class,CONTEXTID);
 
 /*********************************************************************/
 
-void DeleteClassFromHeap(class)
-
-char *class;
+void DeleteClassFromHeap(char *class)
 
 {
 DeleteItemLiteral(&VHEAP,class);
@@ -326,9 +403,7 @@ DeleteItemLiteral(&VHEAP,class);
 
 /*********************************************************************/
 
-void DeleteClassesFromContext(context)
-
-char *context;
+void DeleteClassesFromContext(char *context)
 
 { struct Item *ip;
 
@@ -346,9 +421,7 @@ for (ip = VHEAP; ip != NULL; ip=ip->next)
 
 /*********************************************************************/
 
-int IsHardClass(sp)  /* true if string matches a hardwired class e.g. hpux */
-
-char *sp;
+int IsHardClass(char *sp)  /* true if string matches a hardwired class e.g. hpux */
 
 { int i;
 
@@ -373,9 +446,7 @@ return(false);
 
 /*******************************************************************/
 
-int IsSpecialClass(class)
-
-char *class;
+int IsSpecialClass(char *class)
 
 { int value = -1;
 
@@ -421,9 +492,7 @@ return false;
 
 /*********************************************************************/
 
-int IsExcluded(exception)
-
-char *exception;
+int IsExcluded(char *exception)
 
 {
 if (! IsDefinedClass(exception))
@@ -437,14 +506,14 @@ return false;
 
 /*********************************************************************/
 
-int IsDefinedClass(class) 
+int IsDefinedClass(char *class) 
 
   /* Evaluates a.b.c|d.e.f etc and returns true if the class */
   /* is currently true, given the defined heap and negations */
 
-char *class;
-
 {
+ Debug4("IsDefinedClass(%s,VADDCLASSES)\n",class);
+
 if (class == NULL)
    {
    return true;
@@ -456,16 +525,16 @@ return EvaluateORString(class,VADDCLASSES);
 
 /*********************************************************************/
 
-int IsInstallable(class)
-
-char *class;
+int IsInstallable(char *class)
 
   /* Evaluates to true if the class string COULD become true in */
   /* the course of the execution - but might not be true now    */
 
-{ char buffer[bufsize], *sp;
-  int i = 0;
+{ char buffer[CF_BUFSIZE], *sp;
+ int i = 0, val;
 
+Debug1("IsInstallable(%s) -",class);
+  
 for (sp = class; *sp != '\0'; sp++)
    {
    if (*sp == '!')
@@ -477,52 +546,29 @@ for (sp = class; *sp != '\0'; sp++)
 
 buffer[i] = '\0';
  
-return (EvaluateORString(buffer,VALLADDCLASSES)||EvaluateORString(class,VALLADDCLASSES)||EvaluateORString(class,VADDCLASSES));
-}
+/* return (EvaluateORString(buffer,VALLADDCLASSES)||EvaluateORString(class,VADDCLASSES));*/
 
-/*********************************************************************/
+val = (EvaluateORString(buffer,VALLADDCLASSES)||EvaluateORString(class,VALLADDCLASSES)||EvaluateORString(class,VADDCLASSES));
 
-void AddCompoundClass(class)
-
-char *class;
-
-{ char *sp = class;
-  char cbuff[maxvarsize];
-
-Debug1("AddCompoundClass(%s)",class);
-
-while(*sp != '\0')
+if (val)
    {
-   sscanf(sp,"%[^.]",cbuff);
-
-   while ((*sp != '\0') && (*sp !='.'))
-      {
-      sp++;
-      }
-
-   if (*sp == '.')
-      {
-      sp++;
-      }
-
-   if (IsHardClass(cbuff))
-      {
-      FatalError("cfengine: You cannot use -D to define a reserved class!");
-      }
-
-   AddClassToHeap(cbuff);
+   Debug1(" true\n");
    }
+else
+   {
+   Debug1(" false\n");
+   }
+
+return val;
 }
 
+
 /*********************************************************************/
 
-void NegateCompoundClass(class,heap)
-
-char *class;
-struct Item **heap;
+void NegateCompoundClass(char *class,struct Item **heap)
 
 { char *sp = class;
-  char cbuff[maxvarsize];
+  char cbuff[CF_MAXVARSIZE];
 
 Debug1("NegateCompoundClass(%s)",class);
 
@@ -530,18 +576,18 @@ while(*sp != '\0')
    {
    sscanf(sp,"%[^.]",cbuff);
 
-   while ((*sp != '\0') && (*sp !='.'))
+   while ((*sp != '\0') && ((*sp !='.')||(*sp == '&')))
       {
       sp++;
       }
 
-   if (*sp == '.')
+   if ((*sp == '.') || (*sp == '&'))
       {
       sp++;
       }
 
    if (IsHardClass(cbuff))
-      { char err[bufsize];
+      { char err[CF_BUFSIZE];
       yyerror("Illegal exception");
       sprintf (err,"Cannot negate the reserved class [%s]\n",cbuff);
       FatalError(err);
@@ -555,12 +601,9 @@ while(*sp != '\0')
 /* Level 2                                                           */
 /*********************************************************************/
 
-int EvaluateORString(class,list)
+int EvaluateORString(char *class,struct Item *list)
 
-char *class;
-struct Item *list;
-
-{ char *sp, cbuff[bufsize];
+{ char *sp, cbuff[CF_BUFSIZE];
   int result = false;
 
 if (class == NULL)
@@ -568,6 +611,8 @@ if (class == NULL)
    return false;
    }
 
+Debug4("\n--------\nEvaluateORString(%s)\n",class);
+ 
 for (sp = class; *sp != '\0'; sp++)
    {
    while (*sp == '|')
@@ -575,7 +620,7 @@ for (sp = class; *sp != '\0'; sp++)
       sp++;
       }
 
-   bzero(cbuff,bufsize);
+   memset(cbuff,0,CF_BUFSIZE);
 
    sp += GetORAtom(sp,cbuff);
 
@@ -584,15 +629,18 @@ for (sp = class; *sp != '\0'; sp++)
       break;
       }
 
+
    if (IsBracketed(cbuff)) /* Strip brackets */
       {
       cbuff[strlen(cbuff)-1] = '\0';
 
       result |= EvaluateORString(cbuff+1,list);
+      Debug4("EvalORString-temp-result-y=%d (%s)\n",result,cbuff+1);
       }
    else
       {
       result |= EvaluateANDString(cbuff,list);
+      Debug4("EvalORString-temp-result-n=%d (%s)\n",result,cbuff);
       }
 
    if (*sp == '\0')
@@ -601,6 +649,7 @@ for (sp = class; *sp != '\0'; sp++)
       }
    }
 
+Debug4("EvaluateORString(%s) returns %d\n",class,result); 
 return result;
 }
 
@@ -608,19 +657,18 @@ return result;
 /* Level 3                                                           */
 /*********************************************************************/
 
-int EvaluateANDString(class,list)
-
-char *class;
-struct Item *list;
+int EvaluateANDString(char *class,struct Item *list)
 
 { char *sp, *atom;
-  char cbuff[bufsize];
+  char cbuff[CF_BUFSIZE];
   int count = 1;
   int negation = false;
 
+Debug4("EvaluateANDString(%s)\n",class);
+
 count = CountEvalAtoms(class);
 sp = class;
-
+ 
 while(*sp != '\0')
    {
    negation = false;
@@ -631,7 +679,7 @@ while(*sp != '\0')
       sp++;
       }
 
-   bzero(cbuff,bufsize);
+   memset(cbuff,0,CF_BUFSIZE);
 
    sp += GetANDAtom(sp,cbuff) + 1;
 
@@ -643,31 +691,36 @@ while(*sp != '\0')
       {
       atom = cbuff+1;
 
+      Debug4("Checking AND Atom %s?\n");
+      
       cbuff[strlen(cbuff)-1] = '\0';
-
+      
       if (EvaluateORString(atom,list))
-	 {
-	 if (negation)
-	    {
-	    return false;
-	    }
-	 else
-	    {
-	    count--;
-	    }
-	 }
+         {
+         if (negation)
+            {
+            Debug4("EvalANDString-temp-result-neg1=false\n");
+            return false;
+            }
+         else
+            {
+            Debug4("EvalORString-temp-result count=%d\n",count);
+            count--;
+            }
+         }
       else
-	 {
-	 if (negation)
-	    {
-	    count--;
-	    }
-	 else
-	    {
-	    return false;
-	    }
-	 }
-
+         {
+         if (negation)
+            {
+            Debug4("EvalORString-temp-result2 count=%d\n",count);
+            count--;
+            }
+         else
+            {
+            return false;
+            }
+         }
+      
       continue;
       }
    else
@@ -677,30 +730,35 @@ while(*sp != '\0')
    
    /* End of parenthesis check */
    
-   if (*sp == '.')
+   if (*sp == '.' || *sp == '&')
       {
       sp++;
       }
 
+   Debug4("Checking OR atom (%s)?\n",atom);
+
    if (IsItemIn(VNEGHEAP,atom))
       {
       if (negation)
          {
-	 count--;
-	 }
+         Debug4("EvalORString-temp-result3 count=%d\n",count);
+         count--;
+         }
       else
-	 {
-	 return false;
-	 }
+         {
+         return false;
+         }
       } 
    else if (IsItemIn(VHEAP,atom))
       {
       if (negation)
          {
-	 return false;
+         Debug4("EvaluateANDString(%s) returns false by negation\n",class);
+         return false;
          }
       else
          {
+         Debug4("EvalORString-temp-result3.5 count=%d\n",count);
          count--;
          }
       } 
@@ -708,38 +766,43 @@ while(*sp != '\0')
       {
       if (negation)
          {
+         Debug4("EvaluateANDString(%s) returns false by negation 2\n",class);
          return false;
          }
       else
          {
+         Debug4("EvalORString-temp-result3.6 count=%d\n",count);
          count--;
          }
       } 
    else if (negation)    /* ! (an undefined class) == true */
       {
+      Debug4("EvalORString-temp-result4 count=%d\n",count);
       count--;
       }
-   else
+   else       
       {
+      Debug4("EvaluateANDString(%s) returns false ny negation 3\n",class);
       return false;
       }
    }
 
+ 
 if (count == 0)
    {
+   Debug4("EvaluateANDString(%s) returns true\n",class);
    return(true);
    }
 else
    {
+   Debug4("EvaluateANDString(%s) returns false\n",class);
    return(false);
    }
 }
 
 /*********************************************************************/
 
-int GetORAtom(start,buffer)
-
-char *start, *buffer;
+int GetORAtom(char *start,char *buffer)
 
 { char *sp = start;
   char *spc = buffer;
@@ -749,20 +812,24 @@ while ((*sp != '\0') && !((*sp == '|') && (bracklevel == 0)))
    {
    if (*sp == '(')
       {
+      Debug4("+(\n");
       bracklevel++;
       }
 
    if (*sp == ')')
       {
+      Debug4("-)\n");
       bracklevel--;
       }
 
+   Debug4("(%c)",*sp);
    *spc++ = *sp++;
    len++;
    }
 
 *spc = '\0';
 
+Debug4("\nGetORATom(%s)->%s\n",start,buffer); 
 return len;
 }
 
@@ -770,23 +837,23 @@ return len;
 /* Level 4                                                           */
 /*********************************************************************/
 
-int GetANDAtom(start,buffer)
-
-char *start, *buffer;
+int GetANDAtom(char *start,char *buffer)
 
 { char *sp = start;
   char *spc = buffer;
   int bracklevel = 0, len = 0;
 
-while ((*sp != '\0') && !((*sp == '.') && (bracklevel == 0)))
+while ((*sp != '\0') && !(((*sp == '.')||(*sp == '&')) && (bracklevel == 0)))
    {
    if (*sp == '(')
       {
+      Debug4("+(\n");
       bracklevel++;
       }
 
    if (*sp == ')')
       {
+      Debug("-)\n");
       bracklevel--;
       }
 
@@ -796,15 +863,14 @@ while ((*sp != '\0') && !((*sp == '.') && (bracklevel == 0)))
    }
 
 *spc = '\0';
+Debug4("\nGetANDATom(%s)->%s\n",start,buffer);  
 
 return len;
 }
 
 /*********************************************************************/
 
-int CountEvalAtoms(class)
-
-char *class;
+int CountEvalAtoms(char *class)
 
 { char *sp;
   int count = 0, bracklevel = 0;
@@ -813,17 +879,19 @@ for (sp = class; *sp != '\0'; sp++)
    {
    if (*sp == '(')
       {
+      Debug4("+(\n");
       bracklevel++;
       continue;
       }
 
    if (*sp == ')')
       {
+      Debug4("-)\n");
       bracklevel--;
       continue;
       }
    
-   if ((bracklevel == 0) && (*sp == '.'))
+   if ((bracklevel == 0) && ((*sp == '.')||(*sp == '&')))
       {
       count++;
       }
@@ -843,9 +911,7 @@ return count+1;
 /* TOOLKIT : actions                                                 */
 /*********************************************************************/
 
-enum actions ActionStringToCode (str)
-
-char *str;
+enum actions ActionStringToCode (char *str)
 
 { char *sp;
   int i;
@@ -882,13 +948,11 @@ return (enum actions) i;
 
 /*********************************************************************/
 
-int IsBracketed(s)
+int IsBracketed(char *s)
 
  /* return true if the entire string is bracketed, not just if
     if contains brackets */
 
-char *s;
-
 { int i, level= 0;
 
 if (*s != '(')
