@@ -32,10 +32,7 @@
 /* Macro substitution based on HASH-table                          */
 /*******************************************************************/
 
-
-void SetContext(id)
-
-char *id;
+void SetContext(char *id)
 
 {
 InstallObject(id);
@@ -48,13 +45,71 @@ strncpy(CONTEXTID,id,31);
 
 /*******************************************************************/
 
-void InitHashTable(table)
+void InstallObject(char *name)
 
-char **table;
+{ struct cfObject *ptr;
+  
+Debug1("Adding object %s", name);
+
+/*
+  if ( ! IsInstallable(CLASSBUFF))
+   {
+   return;
+   }
+*/
+
+for (ptr = VOBJ; ptr != NULL; ptr=ptr->next)
+   {
+   if (strcmp(ptr->scope,name) == 0)
+      {
+      Debug("Object %s already exists\n",name);
+      return;
+      }
+   }
+
+if ((ptr = (struct cfObject *)malloc(sizeof(struct cfObject))) == NULL)
+   {
+   FatalError("Memory Allocation failed for cfObject");
+   }
+ 
+if (VOBJTOP == NULL)
+   {
+   VOBJ = ptr;
+   }
+else
+   {
+   VOBJTOP->next = ptr;
+   }
+
+InitHashTable(ptr->hashtable);
+ 
+ptr->next = NULL;
+ptr->scope = strdup(name);
+VOBJTOP = ptr; 
+}
+
+/*******************************************************************/
+
+int ScopeIsMethod()
+
+{
+ if (strcmp(CONTEXTID,"private-method") == 0)
+    {
+    return true;
+    }
+ else
+    {
+    return false;
+    }
+}
+
+/*******************************************************************/
+
+void InitHashTable(char **table)
 
 { int i;
 
-for (i = 0; i < hashtablesize; i++)
+for (i = 0; i < CF_HASHTABLESIZE; i++)
    {
    table[i] = NULL;
    }
@@ -62,32 +117,28 @@ for (i = 0; i < hashtablesize; i++)
 
 /*******************************************************************/
 
-void PrintHashTable(table)
-
-char **table;
+void PrintHashTable(char **table)
 
 { int i;
 
-for (i = 0; i < hashtablesize; i++)
+for (i = 0; i < CF_HASHTABLESIZE; i++)
    {
    if (table[i] != NULL)
       {
-      printf ("%d : %s\n",i,table[i]);
+      printf ("   %d : %s\n",i,table[i]);
       }
    }
 }
 
 /*******************************************************************/
 
-int Hash(name)
-
-char *name;
+int Hash(char *name)
 
 { int i, slot = 0;
 
  for (i = 0; name[i] != '\0'; i++)
    {
-   slot = (macroalphabet * slot + name[i]) % hashtablesize;
+   slot = (CF_MACROALPHABET * slot + name[i]) % CF_HASHTABLESIZE;
    }
 
 return slot;
@@ -95,9 +146,7 @@ return slot;
 
 /*******************************************************************/
 
-int ElfHash(key)
-
-char *key;
+int ElfHash(char *key)
 
 { unsigned int h = 0;
   unsigned int g;
@@ -116,16 +165,14 @@ while (*key)
   h &= ~g;
   }
 
-return (h % hashtablesize);
+return (h % CF_HASHTABLESIZE);
 }
 
 /*******************************************************************/
 
-void AddMacroValue(scope,name,value)
+void AddMacroValue(char *scope,char *name,char *value)
 
-char *scope,*name,*value;
-
-{ char *sp, buffer[bufsize],exp[bufsize];
+{ char *sp, buffer[CF_BUFSIZE],exp[CF_EXPANDSIZE];
   struct cfObject *ptr; 
   int slot;
 
@@ -137,7 +184,7 @@ if (name == NULL || value == NULL || scope == NULL)
    CfLog(cferror,"Empty macro","");
    }
 
-if (strlen(name) > maxvarsize)
+if (strlen(name) > CF_MAXVARSIZE)
    {
    yyerror("macro name too long");
    return;
@@ -147,7 +194,7 @@ ExpandVarstring(value,exp,NULL);
 
 ptr = ObjectContext(scope);
  
-snprintf(buffer,bufsize,"%s=%s",name,exp);
+snprintf(buffer,CF_BUFSIZE,"%s=%s",name,exp);
 
 if ((sp = malloc(strlen(buffer)+1)) == NULL)
    {
@@ -165,18 +212,19 @@ if (ptr->hashtable[slot] != 0)
    if (CompareMacro(name,ptr->hashtable[slot]) == 0)
       {
       if (PARSING && !IsItemIn(VREDEFINES,name))
-	 {
-	 snprintf(VBUFF,bufsize,"Redefinition of macro %s=%s",name,exp);
-	 Warning(VBUFF);
-	 }
+         {
+         snprintf(VBUFF,CF_BUFSIZE,"Redefinition of macro %s=%s (or perhaps missing quote)",name,exp);
+         Warning(VBUFF);
+         }
       free(ptr->hashtable[slot]);
       ptr->hashtable[slot] = sp;
+      Debug("Stored %s in context %s\n",sp,name);
       return;
       }
-
-   while ((ptr->hashtable[++slot % hashtablesize] != 0))
+   
+   while ((ptr->hashtable[++slot % CF_HASHTABLESIZE] != 0))
       {
-      if (slot == hashtablesize-1)
+      if (slot == CF_HASHTABLESIZE-1)
          {
          slot = 0;
          }
@@ -184,56 +232,111 @@ if (ptr->hashtable[slot] != 0)
          {
          FatalError("AddMacroValue - internal error #1");
          }
-
+      
       if (CompareMacro(name,ptr->hashtable[slot]) == 0)
-	 {
-	 snprintf(VBUFF,bufsize,"Redefinition of macro %s=%s",name,exp);
-	 Warning(VBUFF);
-	 free(ptr->hashtable[slot]);
-	 ptr->hashtable[slot] = sp;
-	 return;
-	 }
+         {
+         snprintf(VBUFF,CF_BUFSIZE,"Redefinition of macro %s=%s",name,exp);
+         Warning(VBUFF);
+         free(ptr->hashtable[slot]);
+         ptr->hashtable[slot] = sp;
+         return;
+         }
       }
    }
-
-ptr->hashtable[slot] = sp;
+ 
+ ptr->hashtable[slot] = sp;
 
 Debug("Added Macro at hash address %d to object %s with value %s\n",slot,scope,sp);
 }
 
+/****************************************************************************/
+
+/*
+ * HvB: Bas van der Vlies
+ *  This function checks if the given name has
+ *  the requested value:
+ *    1 --> check for values on or true
+ *    0 --> check for values off or false
+ *  return true if the name has the requested value
+*/
+
+int OptionIs(char *scope,char *name,short on)
+
+{ char *result;
+
+Debug("OptionIs(%s,%s,%d)\n",scope,name,on);
+ 
+result = GetMacroValue(scope,name);
+
+if (result == NULL)
+   {
+   return(false);
+   }
+
+if (on)
+   {
+   if ( (strcmp(result, "on") == 0) || ( strcmp(result, "true") == 0) )
+      {
+      return(true);
+      }
+   }
+else
+   {
+   if ( (strcmp(result, "off") == 0) || (strcmp(result, "false") == 0) )
+      {
+      return(true);
+      }
+   }
+
+return(false);
+}
+
+
 /*******************************************************************/
 
-char *GetMacroValue(scope,name)
-
-char *scope,*name;
+char *GetMacroValue(char *scope,char *name)
 
 { char *sp;
   int slot,i;
-  struct cfObject *ptr;
+  struct cfObject *ptr = NULL;
+  char objscope[CF_MAXVARSIZE],vname[CF_MAXVARSIZE];
   
 /* Check the class.id identity to see if this is private ..
     and replace
 
      HASH[slot] by objectptr->hash[slot]
-
 */
 
-  
-ptr = ObjectContext(scope);	     
-i = slot = Hash(name);
+Debug("GetMacroValue(%s,%s)\n",scope,name);
+
+if (strstr(name,"."))
+   {
+   objscope[0] = '\0';
+   sscanf(name,"%[^.].%s",objscope,vname);
+   Debug("Macro identifier %s is prefixed with an object %s\n",vname,objscope);
+   ptr = ObjectContext(objscope);
+   }
  
-if (CompareMacro(name,ptr->hashtable[slot]) != 0)
+if (ptr == NULL)
+   {
+   strcpy(vname,name);
+   ptr = ObjectContext(scope);
+   }
+ 
+i = slot = Hash(vname);
+ 
+if (CompareMacro(vname,ptr->hashtable[slot]) != 0)
    {
    while (true)
       {
       i++;
 
-      if (i >= hashtablesize-1)
+      if (i >= CF_HASHTABLESIZE-1)
          {
          i = 0;
          }
 
-      if (CompareMacro(name,ptr->hashtable[i]) == 0)
+      if (CompareMacro(vname,ptr->hashtable[i]) == 0)
          {
          for(sp = ptr->hashtable[i]; *sp != '='; sp++)
             {
@@ -244,7 +347,7 @@ if (CompareMacro(name,ptr->hashtable[slot]) != 0)
 
       if (i == slot-1)
          {
-         return(getenv(name));  /* Look in environment if not found */
+         return(getenv(vname));  /* Look in environment if not found */
          }
       }
    }
@@ -260,9 +363,7 @@ else
 
 /*******************************************************************/
 
-void DeleteMacro(scope,id)
-
-char *id, *scope;
+void DeleteMacro(char *scope,char *id)
 
 { int slot,i;
   struct cfObject *ptr;
@@ -275,40 +376,38 @@ if (CompareMacro(id,ptr->hashtable[slot]) != 0)
    while (true)
       {
       i++;
-
-      if (i == slot)
-	 {
-	 Debug("No macro matched\n");
-	 break;
-	 }
       
-      if (i >= hashtablesize-1)
+      if (i == slot)
+         {
+         Debug("No macro matched\n");
+         break;
+         }
+      
+      if (i >= CF_HASHTABLESIZE-1)
          {
          i = 0;
          }
-
+      
       if (CompareMacro(id,ptr->hashtable[i]) == 0)
          {
          free(ptr->hashtable[i]);
-	 ptr->hashtable[i] = NULL;
+         ptr->hashtable[i] = NULL;
          }
       }
    }
-else
-   {
-   free(ptr->hashtable[i]);
-   ptr->hashtable[i] = NULL;
-   }   
+ else
+    {
+    free(ptr->hashtable[i]);
+    ptr->hashtable[i] = NULL;
+    }   
 }
 
 
 /*******************************************************************/
 
-int CompareMacro(name,macro)
+int CompareMacro(char *name,char *macro)
 
-char *name,*macro;
-
-{ char buffer[bufsize];
+{ char buffer[CF_BUFSIZE];
 
 if (macro == NULL || name == NULL)
    {
@@ -316,23 +415,19 @@ if (macro == NULL || name == NULL)
    }
 
 sscanf(macro,"%[^=]",buffer);
-
-Debug1("CompareMacro(%s,%s)=%s\n",name,macro,buffer);
 return(strcmp(buffer,name));
 }
 
 /*******************************************************************/
 
-void DeleteMacros(scope)
-
-char *scope;
+void DeleteMacros(char *scope)
 
 { int i;
   struct cfObject *ptr;
   
 ptr = ObjectContext(scope);
  
-for (i = 0; i < hashtablesize; i++)
+for (i = 0; i < CF_HASHTABLESIZE; i++)
    {
    if (ptr->hashtable[i] != NULL)
       {
@@ -344,9 +439,7 @@ for (i = 0; i < hashtablesize; i++)
 
 /*******************************************************************/
 
-struct cfObject *ObjectContext(scope)
-
-char *scope;
+struct cfObject *ObjectContext(char *scope)
 
 { struct cfObject *cp = NULL;
 
