@@ -39,15 +39,12 @@
 /* TOOLKIT : files/directories                                       */
 /*********************************************************************/
 
-int DirPush(name,sb)          /* Enter dir and check for race exploits */
-
-char *name;
-struct stat *sb;
+int DirPush(char *name,struct stat *sb)          /* Enter dir and check for race exploits */
 
 {
 if (chdir(name) == -1)
    {
-   snprintf(OUTPUT,bufsize,"Could not change to directory %s, mode %o in tidy",name,sb->st_mode & 07777);
+   snprintf(OUTPUT,CF_BUFSIZE,"Could not change to directory %s, mode %o in tidy",name,sb->st_mode & 07777);
    CfLog(cfinform,OUTPUT,"chdir");
    return false;
    }
@@ -62,18 +59,14 @@ return true;
 
 /**********************************************************************/
 
-void DirPop(goback,name,sb)      /* Exit dir and check for race exploits */
-
-char *name;
-int goback;
-struct stat *sb;
+void DirPop(int goback,char * name,struct stat *sb)      /* Exit dir and check for race exploits */
 
 {
 if (goback && TRAVLINKS)
    {
    if (chdir(name) == -1)
       {
-      snprintf(OUTPUT,bufsize,"Error in backing out of recursive descent securely to %s",name);
+      snprintf(OUTPUT,CF_BUFSIZE,"Error in backing out of recursive descent securely to %s",name);
       CfLog(cferror,OUTPUT,"chdir");
       HandleSignal(SIGTERM);
       }
@@ -84,7 +77,7 @@ else if (goback)
    {
    if (chdir("..") == -1)
       {
-      snprintf(OUTPUT,bufsize,"Error in backing out of recursive descent securely to %s",name);
+      snprintf(OUTPUT,CF_BUFSIZE,"Error in backing out of recursive descent securely to %s",name);
       CfLog(cferror,OUTPUT,"chdir");
       HandleSignal(SIGTERM);
       }
@@ -93,10 +86,7 @@ else if (goback)
 
 /**********************************************************************/
 
-void CheckLinkSecurity(sb,name)
-
-struct stat *sb;
-char *name;
+void CheckLinkSecurity(struct stat *sb,char *name)
 
 { struct stat security;
 
@@ -104,14 +94,14 @@ Debug("Checking the inode and device to make sure we are where we think we are..
 
 if (stat(".",&security) == -1)
    {
-   snprintf(OUTPUT,bufsize,"Could not stat directory %s after entering!",name);
+   snprintf(OUTPUT,CF_BUFSIZE,"Could not stat directory %s after entering!",name);
    CfLog(cferror,OUTPUT,"stat");
    return;
    }
 
 if ((sb->st_dev != security.st_dev) || (sb->st_ino != security.st_ino))
    {
-   snprintf(OUTPUT,bufsize,"SERIOUS SECURITY ALERT: path race exploited in recursion to/from %s. Not safe for agent to continue - aborting",name);
+   snprintf(OUTPUT,CF_BUFSIZE,"SERIOUS SECURITY ALERT: path race exploited in recursion to/from %s. Not safe for agent to continue - aborting",name);
    CfLog(cferror,OUTPUT,"");
    HandleSignal(SIGTERM);
    /* Exits */
@@ -120,9 +110,7 @@ if ((sb->st_dev != security.st_dev) || (sb->st_ino != security.st_ino))
 
 /**********************************************************************/
 
-void TruncateFile(name)
-
-char *name;
+void TruncateFile(char *name)
 
 { struct stat statbuf;
   int fd;
@@ -136,7 +124,7 @@ else
    {
    if ((fd = creat(name,000)) == -1)      /* dummy mode ignored */
       {
-      snprintf(OUTPUT,bufsize*2,"creat(%s) failed\n",name);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"creat(%s) failed\n",name);
       CfLog(cferror,OUTPUT,"creat");
       }
    else
@@ -148,9 +136,7 @@ else
 
 /*************************************************************************/
 
-int FileSecure (name)
-
-char *name;
+int FileSecure (char *name)
 
 { struct stat statbuf;
 
@@ -166,7 +152,7 @@ if (stat(name,&statbuf) == -1)
 
 if (statbuf.st_uid != getuid())
    {
-   snprintf(OUTPUT,bufsize*2,"File %s is not owned by uid %d (security exception)",name,getuid());
+   snprintf(OUTPUT,CF_BUFSIZE*2,"File %s is not owned by uid %d (security exception)",name,getuid());
    CfLog(cferror,OUTPUT,"");
    }
  
@@ -174,7 +160,7 @@ if (statbuf.st_uid != getuid())
  
 if (statbuf.st_mode & (S_IWGRP | S_IWOTH))
    {
-   snprintf(OUTPUT,bufsize*2,"File %s (owner %d) is writable by others (security exception)",name,getuid());
+   snprintf(OUTPUT,CF_BUFSIZE*2,"File %s (owner %d) is writable by others (security exception)",name,getuid());
    CfLog(cferror,OUTPUT,"");
    return false;
    }
@@ -184,15 +170,11 @@ return true;
 
 /***************************************************************/
 
-int ChecksumChanged(filename,digest,warnlevel,refresh,type)
+int ChecksumChanged(char *filename,unsigned char digest[EVP_MAX_MD_SIZE+1],int warnlevel,int refresh,char type)
 
     /* Returns false if filename never seen before, and adds a checksum
        to the database. Returns true if checksums do not match and also
        updates database to the new value */
-    
-char *filename, type;
-unsigned char digest[EVP_MAX_MD_SIZE+1];
-int warnlevel,refresh;
 
 { struct stat stat1, stat2;
   int i,needupdate = false, size = 21;
@@ -224,50 +206,218 @@ if (CHECKSUMDB == NULL)
       ChecksumFile(filename,dbdigest,type);
       
       for (i = 0; i < size; i++)
-	 {
-	 if (digest[i] != dbdigest[i])
-	    {
-	    return true;
-	    }
-	 }
+         {
+         if (digest[i] != dbdigest[i])
+            {
+            return true;
+            }
+         }
       return false;
       }
    }
-
-if (refresh)
-   {
-   /* Check whether database is current wrt local file */
-
-   if (stat(filename,&stat1) == -1)
-      {
-      snprintf(OUTPUT,bufsize*2,"Couldn't stat %s\n",filename);
-      CfLog(cferror,OUTPUT,"stat");
-      return false;
-      }
-   
-   if (stat(CHECKSUMDB,&stat2) != -1)
-      {
-      if (stat1.st_mtime > stat2.st_mtime)
-	 {
-	 Debug("Checksum database is older than %s...refresh needed\n",filename);
-	 needupdate = true;
-	 }
-      else
-	 {
-	 Debug("Checksum up to date..\n");
-	 }
-      }
-   else
-      {
-      needupdate = true;
-      }      
-   }
  
+ if (refresh)
+    {
+    /* Check whether database is current wrt local file */
+    
+    if (stat(filename,&stat1) == -1)
+       {
+       snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't stat %s\n",filename);
+       CfLog(cferror,OUTPUT,"stat");
+       return false;
+       }
+    
+    if (stat(CHECKSUMDB,&stat2) != -1)
+       {
+       if (stat1.st_mtime > stat2.st_mtime)
+          {
+          Debug("Checksum database is older than %s...refresh needed\n",filename);
+          needupdate = true;
+          }
+       else
+          {
+          Debug("Checksum up to date..\n");
+          }
+       }
+    else
+       {
+       needupdate = true;
+       }      
+    }
+ 
+ if ((errno = db_create(&dbp,dbenv,0)) != 0)
+    {
+    snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't open checksum database %s\n",CHECKSUMDB);
+    CfLog(cferror,OUTPUT,"db_open");
+    return false;
+    }
+ 
+#ifdef CF_OLD_DB
+ if ((errno = dbp->open(dbp,CHECKSUMDB,NULL,DB_BTREE,DB_CREATE,0644)) != 0)
+#else
+     if ((errno = dbp->open(dbp,NULL,CHECKSUMDB,NULL,DB_BTREE,DB_CREATE,0644)) != 0)
+#endif
+        {
+        snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't open checksum database %s\n",CHECKSUMDB);
+        CfLog(cferror,OUTPUT,"db_open");
+        dbp->close(dbp,0);
+        return false;
+        }
+ 
+ memset(&value,0,sizeof(value)); 
+ memset(&key,0,sizeof(key));       
+ 
+ key.data = filename;
+ key.size = strlen(filename)+1;
+ value.data = dbvalue;
+ value.size = size; 
+ 
+ if (needupdate)
+    {
+    memset(dbdigest,0,EVP_MAX_MD_SIZE+1);
+    ChecksumFile(filename,dbdigest,type);
+    
+    key.data = filename;
+    key.size = strlen(filename)+1;
+    value.data = (void *) dbdigest;
+    value.size = size;
+    
+    Debug("Updating checksum for %s to %s\n",filename,ChecksumPrint(type,value.data));
+    
+    if ((errno = dbp->del(dbp,NULL,&key,0)) != 0)
+       {
+       CfLog(cferror,"","db_store");
+       }
+    
+    key.data = filename;
+    key.size = strlen(filename)+1;
+    
+    if ((errno = dbp->put(dbp,NULL,&key,&value,0)) != 0)
+       {
+       CfLog(cferror,"put failed","db->put");
+       }      
+    }
+ 
+ if ((errno = dbp->get(dbp,NULL,&key,&value,0)) == 0)
+    {
+    /* The filename key was found in the db */
+    Debug("Comparing %s (sent)\n",ChecksumPrint(type,digest));
+    Debug("     with %s (db)\n",ChecksumPrint(type,value.data));
+    
+    memset(dbdigest,0,EVP_MAX_MD_SIZE+1);
+    memcpy(dbdigest,value.data,size);
+    
+    for (i = 0; i < size; i++)
+       {
+       if (digest[i] != dbdigest[i])
+          {
+          Debug("Found checksum for %s in database but it didn't match\n",filename);
+          
+          if (EXCLAIM)
+             {
+             CfLog(warnlevel,"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!","");
+             }
+          
+          snprintf(OUTPUT,CF_BUFSIZE*2,"SECURITY ALERT: Checksum for %s changed!",filename);
+          CfLog(warnlevel,OUTPUT,"");
+          
+          if (EXCLAIM)
+             {
+             CfLog(warnlevel,"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!","");
+             }
+          
+          if (CHECKSUMUPDATES)
+             {
+             memset(dbdigest,0,EVP_MAX_MD_SIZE+1);
+             ChecksumFile(filename,dbdigest,type);
+             
+             key.data = filename;
+             key.size = strlen(filename)+1;
+             value.data = (void *) dbdigest;
+             value.size = size;
+             
+             Verbose("Updating checksum for %s to %s\n",filename,ChecksumPrint(type,value.data));
+             
+             if ((errno = dbp->del(dbp,NULL,&key,0)) != 0)
+                {
+                CfLog(cferror,"","db->del");
+                }
+             
+             key.data = filename;
+             key.size = strlen(filename)+1;
+             
+             if ((errno = dbp->put(dbp,NULL,&key,&value,0)) != 0)
+                {
+                CfLog(cferror,"put failed","db->put");
+                }
+             }
+          
+          dbp->close(dbp,0);
+          return true;                        /* Checksum updated but was changed */
+          }
+       }
+    
+    Debug("Found checksum for %s in database and it matched\n",filename);
+    dbp->close(dbp,0);
+    return false;
+    }
+ else
+    {
+    /* Key was not found, so install it */
+    
+    if (ISCFENGINE)
+       {
+       snprintf(OUTPUT,CF_BUFSIZE,"File %s was not in md5 database - new file found",filename);
+       CfLog(cfsilent,OUTPUT,"");
+       }
+    
+    memset(dbdigest,0,EVP_MAX_MD_SIZE+1);
+    ChecksumFile(filename,dbdigest,type);
+    
+    key.data = filename;
+    key.size = strlen(filename)+1;
+    value.data = (void *) dbdigest;
+    value.size = size;
+    
+    Debug("Storing checksum for %s in database %s\n",filename,ChecksumPrint(type,dbdigest));
+    
+    if ((errno = dbp->put(dbp,NULL,&key,&value,0)) != 0)
+       {
+       CfLog(cferror,"put failed","db->put");
+       }
+    
+    dbp->close(dbp,0);
+    
+    if (ISCFENGINE)
+       {
+       return false;      /* No need to warn when first installed */
+       }
+    else
+       {
+       return true;
+       }
+    }
+}
+
+
+/***************************************************************/
+
+void ChecksumPurge()
+
+/* Go through the database and purge records about non-existent files */
+
+{ DBT key,value;
+  DB *dbp;
+  DBC *dbcp;
+  DB_ENV *dbenv = NULL;
+  int ret;
+  struct stat statbuf;
+  
 if ((errno = db_create(&dbp,dbenv,0)) != 0)
    {
-   snprintf(OUTPUT,bufsize*2,"Couldn't open checksum database %s\n",CHECKSUMDB);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't open checksum database %s\n",CHECKSUMDB);
    CfLog(cferror,OUTPUT,"db_open");
-   return false;
+   return;
    }
 
 #ifdef CF_OLD_DB
@@ -276,154 +426,50 @@ if ((errno = dbp->open(dbp,CHECKSUMDB,NULL,DB_BTREE,DB_CREATE,0644)) != 0)
 if ((errno = dbp->open(dbp,NULL,CHECKSUMDB,NULL,DB_BTREE,DB_CREATE,0644)) != 0)
 #endif
    {
-   snprintf(OUTPUT,bufsize*2,"Couldn't open checksum database %s\n",CHECKSUMDB);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't open checksum database %s\n",CHECKSUMDB);
    CfLog(cferror,OUTPUT,"db_open");
    dbp->close(dbp,0);
-   return false;
+   return;
    }
 
-bzero(&value,sizeof(value)); 
-bzero(&key,sizeof(key));       
-      
-key.data = filename;
-key.size = strlen(filename)+1;
-value.data = dbvalue;
-value.size = size; 
+/* Acquire a cursor for the database. */
 
-if (needupdate)
-   {
-   bzero(dbdigest,EVP_MAX_MD_SIZE+1);
-   ChecksumFile(filename,dbdigest,type);
-   
-   key.data = filename;
-   key.size = strlen(filename)+1;
-   value.data = (void *) dbdigest;
-   value.size = size;
-   
-   Debug("Updating checksum for %s to %s\n",filename,ChecksumPrint(type,value.data));
-   
-   if ((errno = dbp->del(dbp,NULL,&key,0)) != 0)
-      {
-      CfLog(cferror,"","db_store");
-      }
-  
-   key.data = filename;
-   key.size = strlen(filename)+1;
-   
-   if ((errno = dbp->put(dbp,NULL,&key,&value,0)) != 0)
-      {
-      CfLog(cferror,"put failed","db->put");
-      }      
-   }
+ if ((ret = dbp->cursor(dbp, NULL, &dbcp, 0)) != 0)
+    {
+    CfLog(cferror,"Error reading from checksum database","");
+    dbp->err(dbp, ret, "DB->cursor");
+    return;
+    }
 
-if ((errno = dbp->get(dbp,NULL,&key,&value,0)) == 0)
-   {
-   /* The filename key was found in the db */
-   Debug("Comparing %s (sent)\n",ChecksumPrint(type,digest));
-   Debug("     with %s (db)\n",ChecksumPrint(type,value.data));
+ /* Initialize the key/data return pair. */
 
-   bzero(dbdigest,EVP_MAX_MD_SIZE+1);
-   bcopy(value.data,dbdigest,size);
-   
-   for (i = 0; i < size; i++)
-      {
-      if (digest[i] != dbdigest[i])
-	 {
-	 Debug("Found checksum for %s in database but it didn't match\n",filename);
+ memset(&key,0,sizeof(key));
+ memset(&value,0,sizeof(value));
+ 
+ /* Walk through the database and print out the key/data pairs. */
 
-	 if (EXCLAIM)
-	    {
-	    CfLog(warnlevel,"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!","");
-	    }
+ while (dbcp->c_get(dbcp, &key, &value, DB_NEXT) == 0)
+    {
+    if (stat(key.data,&statbuf) == -1)
+       {
+       Verbose("Purging file %s from checksum db - no longer exists\n",(char *)key.data);
+       snprintf(OUTPUT,CF_BUFSIZE*2,"SECURITY INFO: Checksum for %s purged - file no longer exists!",key.data);
+       CfLog(cferror,OUTPUT,"");
 
-	 snprintf(OUTPUT,bufsize*2,"SECURITY ALERT: Checksum for %s changed!",filename);
-	 CfLog(warnlevel,OUTPUT,"");
-
-	 if (EXCLAIM)
-	    {
-	    CfLog(warnlevel,"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!","");
-	    }
-
-	 if (CHECKSUMUPDATES)
-	    {
-	    bzero(dbdigest,EVP_MAX_MD_SIZE+1);
-	    ChecksumFile(filename,dbdigest,type);
-	    
-	    key.data = filename;
-	    key.size = strlen(filename)+1;
-	    value.data = (void *) dbdigest;
-	    value.size = size;
-	    
-	    Verbose("Updating checksum for %s to %s\n",filename,ChecksumPrint(type,value.data));
-	    
-	    if ((errno = dbp->del(dbp,NULL,&key,0)) != 0)
-	       {
-	       CfLog(cferror,"","db->del");
-	       }
-	    
-	    key.data = filename;
-	    key.size = strlen(filename)+1;
-	    
-	    if ((errno = dbp->put(dbp,NULL,&key,&value,0)) != 0)
-	       {
-	       CfLog(cferror,"put failed","db->put");
-	       }
-	    }
-
-	 dbp->close(dbp,0);
-	 return true;                        /* Checksum updated but was changed */
-	 }
-      }
-   
-   Debug("Found checksum for %s in database and it matched\n",filename);
-   dbp->close(dbp,0);
-   return false;
-   }
-else
-   {
-   /* Key was not found, so install it */
-
-   if (ISCFENGINE)
-      {
-      snprintf(OUTPUT,bufsize,"File %s was not in md5 database - new file found",filename);
-      CfLog(cfsilent,OUTPUT,"");
-      }
-   
-   bzero(dbdigest,EVP_MAX_MD_SIZE+1);
-   ChecksumFile(filename,dbdigest,type);
-
-   key.data = filename;
-   key.size = strlen(filename)+1;
-   value.data = (void *) dbdigest;
-   value.size = size;
-
-   Debug("Storing checksum for %s in database %s\n",filename,ChecksumPrint(type,dbdigest));
-
-   if ((errno = dbp->put(dbp,NULL,&key,&value,0)) != 0)
-      {
-      CfLog(cferror,"put failed","db->put");
-      }
-   
-   dbp->close(dbp,0);
-
-   if (ISCFENGINE)
-      {
-      return false;      /* No need to warn when first installed */
-      }
-   else
-      {
-      return true;
-      }
-   }
+       if ((errno = dbp->del(dbp,NULL,&key,0)) != 0)
+          {
+          CfLog(cferror,"","db_store");
+          }
+       }
+    }
+ 
+dbcp->c_close(dbcp);
+dbp->close(dbp,0);
 }
 
 /*************************************************************************/
 
-int IgnoredOrExcluded(action,file,inclusions,exclusions)
-
-enum actions action;
-char *file;
-struct Item *inclusions, *exclusions;
+int IgnoredOrExcluded(enum actions action,char *file,struct Item *inclusions,struct Item *exclusions)
 
 { char *lastnode;
 
@@ -442,7 +488,7 @@ else
    lastnode = file;
    }
 
-if (inclusions != NULL && !IsWildItemIn(inclusions,lastnode))
+if ((inclusions != NULL) && !IsWildItemIn(inclusions,lastnode))
    {
    Debug("cfengine: skipping non-included pattern %s\n",file);
    return true;
@@ -476,9 +522,7 @@ return false;
 
 /*********************************************************************/
 
-void Banner(string)
-
-char *string;
+void Banner(char *string)
 
 {
 Verbose("---------------------------------------------------------------------\n");
@@ -488,10 +532,7 @@ Verbose("---------------------------------------------------------------------\n
 
 /*********************************************************************/
 
-void DebugBinOut(buffer,len)
-
-char *buffer;
-int len;
+void DebugBinOut(char *buffer,int len)
 
 { char *sp;
 
@@ -507,10 +548,7 @@ Debug("]\n");
 
 /*********************************************************************/
 
-char *ChecksumPrint(type,digest)
-
-char type;
-unsigned char digest[EVP_MAX_MD_SIZE+1];
+char *ChecksumPrint(char type,unsigned char digest[EVP_MAX_MD_SIZE+1])
 
 { unsigned int i;
   static char buffer[EVP_MAX_MD_SIZE*4];
@@ -536,27 +574,25 @@ return buffer;
 
 /*******************************************************************/
 
-int ShellCommandReturnsZero(comm)
-
-char *comm;
+int ShellCommandReturnsZero(char *comm)
 
 { int status, i, argc;
   pid_t pid;
-  char arg[maxshellargs][bufsize];
+  char arg[CF_MAXSHELLARGS][CF_BUFSIZE];
   char **argv;
 
 /* Build argument array */
 
-for (i = 0; i < maxshellargs; i++)
+for (i = 0; i < CF_MAXSHELLARGS; i++)
    {
-   bzero (arg[i],bufsize);
+   memset (arg[i],0,CF_BUFSIZE);
    }
 
 argc = SplitCommand(comm,arg);
 
 if (argc == -1)
    {
-   snprintf(OUTPUT,bufsize,"Too many arguments in %s\n",comm);
+   snprintf(OUTPUT,CF_BUFSIZE,"Too many arguments in %s\n",comm);
    CfLog(cferror,OUTPUT,"");
    return false;
    }
@@ -590,63 +626,96 @@ else if (pid == 0)                     /* child */
    }
 else                                    /* parent */
    {
-   if (wait(&status) != pid)
+   pid_t wait_result;
+   
+   while ((wait_result = wait(&status)) != pid)
       {
-      snprintf(OUTPUT,bufsize,"Wait for child failed\n");
-      CfLog(cfinform,OUTPUT,"wait");
+      if (wait_result <= 0)
+         {
+         snprintf(OUTPUT,CF_BUFSIZE,"Wait for child failed\n");
+         CfLog(cfinform,OUTPUT,"wait");
+         return false;
+         }
+      }
+
+   if (WIFSIGNALED(status))
+      {
+      Debug("Script %s returned: %d\n",comm,WTERMSIG(status));
       return false;
       }
+   
+   if (! WIFEXITED(status))
+      {
+      return false;
+      }
+   
+   if (WEXITSTATUS(status) == 0)
+      {
+      Debug("Shell command returned 0\n");
+      return true;
+      }
    else
       {
-      if (WIFSIGNALED(status))
-         {
-         Debug("Script %s returned: %d\n",comm,WTERMSIG(status));
-         return false;
-         }
-
-      if (! WIFEXITED(status))
-         {
-         return false;
-         }
-
-      if (WEXITSTATUS(status) == 0)
-         {
-	 Debug("Shell command returned 0\n");
-         return true;
-         }
-      else
-         {
-	 Debug("Shell command was non-zero\n");
-         return false;
-         }
+      Debug("Shell command was non-zero\n");
+      return false;
       }
    }
 
 return false;
 }
 
+
+/**********************************************************************/
+
+void SetSignals()
+
+
+{ int i;
+
+ SIGNALS[SIGHUP] = strdup("SIGHUP");
+ SIGNALS[SIGINT] = strdup("SIGINT");
+ SIGNALS[SIGTRAP] = strdup("SIGTRAP");
+ SIGNALS[SIGKILL] = strdup("SIGKILL");
+ SIGNALS[SIGPIPE] = strdup("SIGPIPE");
+ SIGNALS[SIGCONT] = strdup("SIGCONT");
+ SIGNALS[SIGABRT] = strdup("SIGABRT");
+ SIGNALS[SIGSTOP] = strdup("SIGSTOP");
+ SIGNALS[SIGQUIT] = strdup("SIGQUIT");
+ SIGNALS[SIGTERM] = strdup("SIGTERM");
+ SIGNALS[SIGCHLD] = strdup("SIGCHLD");
+ SIGNALS[SIGUSR1] = strdup("SIGUSR1");
+ SIGNALS[SIGUSR2] = strdup("SIGUSR2");
+ SIGNALS[SIGBUS] = strdup("SIGBUS");
+ SIGNALS[SIGSEGV] = strdup("SIGSEGV");
+
+ for (i = 0; i < highest_signal; i++)
+    {
+    if (SIGNALS[i] == NULL)
+       {
+       SIGNALS[i] = strdup("NOSIG");
+       }
+    }
+}
+
 /*********************************************************************/
 
-void SetClassesOnScript(execstr,classes,elseclasses,useshell)
-
-char *execstr, *classes, *elseclasses;
-int useshell;
+void SetClassesOnScript(char *execstr,char *classes,char *elseclasses,int useshell)
 
 { FILE *pp;
   int print;
-  char line[bufsize],*sp;
+  char line[CF_BUFSIZE],*sp;
  
 switch (useshell)
    {
    case 'y':  pp = cfpopen_sh(execstr,"r");
               break;
    default:   pp = cfpopen(execstr,"r");
-              break;	     
+              break;      
    }
  
 if (pp == NULL)
    {
-   snprintf(OUTPUT,bufsize*2,"Couldn't open pipe to command %s\n",execstr);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't open pipe to command %s\n",execstr);
    CfLog(cferror,OUTPUT,"popen");
    return;
    } 
@@ -655,12 +724,12 @@ while (!feof(pp))
    {
    if (ferror(pp))  /* abortable */
       {
-      snprintf(OUTPUT,bufsize*2,"Shell command pipe %s\n",execstr);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Shell command pipe %s\n",execstr);
       CfLog(cferror,OUTPUT,"ferror");
       break;
       }
    
-   ReadLine(line,bufsize,pp);
+   ReadLine(line,CF_BUFSIZE,pp);
    
    if (strstr(line,"cfengine-die"))
       {
@@ -669,7 +738,7 @@ while (!feof(pp))
    
    if (ferror(pp))  /* abortable */
       {
-      snprintf(OUTPUT,bufsize*2,"Shell command pipe %s\n",execstr);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Shell command pipe %s\n",execstr);
       CfLog(cferror,OUTPUT,"ferror");
       break;
       }
@@ -683,10 +752,10 @@ while (!feof(pp))
    for (sp = line; *sp != '\0'; sp++)
       {
       if (! isspace((int)*sp))
-	 {
-	 print = true;
-	 break;
-	 }
+         {
+         print = true;
+         break;
+         }
       }
    
    if (print)
@@ -694,15 +763,13 @@ while (!feof(pp))
       Verbose("%s:%s: %s\n",VPREFIX,execstr,line);
       }
    }
-
+ 
 cfpclose_def(pp,classes,elseclasses);
 }
 
 /*********************************************************************/
 
-char *UnQuote(s)
-
-char *s;
+char *UnQuote(char *s)
 
 {
  if (s[strlen(s)-1] == '\"')
@@ -720,13 +787,189 @@ char *s;
     }
 }
 
+/*********************************************************************/
+
+void AddListSeparator(char *s)
+
+{
+if (s[strlen(s)-1] != LISTSEPARATOR)
+   {
+   s[strlen(s)+1] = '\0';
+   s[strlen(s)] = LISTSEPARATOR;
+   }
+}
+
+/*********************************************************************/
+
+void ChopListSeparator(char *s)
+
+{
+if (s[strlen(s)-1] == LISTSEPARATOR)
+   {
+   s[strlen(s)-1] = '\0';
+   } 
+}
+
+/*******************************************************************/
+
+void IDClasses()
+
+{ struct stat statbuf;
+  char *sp;
+  int i = 0;
+ 
+AddClassToHeap("any");      /* This is a reserved word / wildcard */
+ 
+snprintf(VBUFF,CF_BUFSIZE,"cfengine_%s",CanonifyName(VERSION));
+AddClassToHeap(VBUFF);
+ 
+for (sp = VBUFF+strlen(VBUFF); i < 2; sp--)
+   {
+   if (*sp == '_')
+      {
+      i++;
+      *sp = '\0';
+      AddClassToHeap(VBUFF);
+      }
+   }
+
+if (stat("/etc/mandrake-release",&statbuf) != -1)
+   {
+   Verbose("This appears to be a mandrake system.\n");
+   AddClassToHeap("Mandrake");
+   linux_mandrake_version();
+   }
+/* Mandrake has a symlink at /etc/redhat-release pointing to
+ * /etc/mandrake-release, so we else-if around that
+ */
+else if (stat("/etc/redhat-release",&statbuf) != -1)
+   {
+   Verbose("This appears to be a redhat system.\n");
+   AddClassToHeap("redhat");
+   linux_redhat_version();
+   }
+
+if (stat("/etc/SuSE-release",&statbuf) != -1)
+   {
+   Verbose("\nThis appears to be a SuSE system.\n");
+   AddClassToHeap("SuSE");
+   linux_suse_version();
+   }
+
+if (stat("/etc/slackware-release",&statbuf) != -1)
+   {
+   Verbose("\nThis appears to be a slackware system.\n");
+   AddClassToHeap("slackware");
+   }
+ 
+if (stat("/etc/debian_version",&statbuf) != -1)
+   {
+   Verbose("\nThis appears to be a debian system.\n");
+   AddClassToHeap("debian");
+   debian_version();
+   }
+
+if (stat("/etc/UnitedLinux-release",&statbuf) != -1)
+   {
+   Verbose("\nThis appears to be a UnitedLinux system.\n");
+   AddClassToHeap("UnitedLinux");
+   }
+ 
+}
+
+/*********************************************************************************/
+
+int linux_fedora_version(void)
+{
+#define FEDORA_ID "Fedora Core"
+
+#define RELEASE_FLAG "release "
+
+/* We are looking for one of the following strings...
+ *
+ * Fedora Core release 1 (Yarrow)
+ */
+
+#define FEDORA_REL_FILENAME "/etc/fedora-release"
+
+FILE *fp;
+
+/* The full string read in from fedora-release */
+char relstring[CF_MAXVARSIZE];
+char classbuf[CF_MAXVARSIZE];
+
+/* Fedora */
+char *vendor="";
+/* Where the numerical release will be found */
+char *release=NULL;
+
+int major = -1;
+char strmajor[CF_MAXVARSIZE];
+
+/* Grab the first line from the file and then close it. */
+ if ((fp = fopen(FEDORA_REL_FILENAME,"r")) == NULL)
+    {
+    return 1;
+    }
+ fgets(relstring, sizeof(relstring), fp);
+ fclose(fp);
+ 
+ Verbose("Looking for fedora core linux info...\n");
+ 
+ /* First, try to grok the vendor */
+ if(!strncmp(relstring, FEDORA_ID, strlen(FEDORA_ID)))
+    {
+    vendor = "fedora";
+    }
+ else
+    {
+    Verbose("Could not identify OS distro from %s\n", FEDORA_REL_FILENAME);
+    return 2;
+    }
+ 
+ /* Now, grok the release.  We assume that all the strings will
+  * have the word 'release' before the numerical release.
+  */
+ release = strstr(relstring, RELEASE_FLAG);
+ if(release == NULL)
+    {
+    Verbose("Could not find a numeric OS release in %s\n",
+     FEDORA_REL_FILENAME);
+    return 2;
+    }
+ else
+    {
+    release += strlen(RELEASE_FLAG);
+    if (sscanf(release, "%d", &major) == 1)
+       {
+       sprintf(strmajor, "%d", major);
+       }
+    }
+ 
+ if (major != -1 && vendor != "")
+    {
+    classbuf[0] = '\0';
+    strcat(classbuf, vendor);
+    AddClassToHeap(classbuf);
+    strcat(classbuf, "_");
+    strcat(classbuf, strmajor);
+    AddClassToHeap(classbuf);
+    }
+ 
+ return 0;
+}
+
 /*********************************************************************************/
 
 int linux_redhat_version(void)
 {
 #define REDHAT_ID "Red Hat Linux"
-#define REDHAT_AS_ID "Red Hat Linux Advanced Server"
+#define REDHAT_AS_ID "Red Hat Enterprise Linux AS"
+#define REDHAT_AS21_ID "Red Hat Linux Advanced Server"
+#define REDHAT_ES_ID "Red Hat Enterprise Linux ES"
+#define REDHAT_WS_ID "Red Hat Enterprise Linux WS"
 #define MANDRAKE_ID "Linux Mandrake"
+#define FEDORA_ID "Fedora Core"
 
 #define RELEASE_FLAG "release "
 
@@ -734,28 +977,32 @@ int linux_redhat_version(void)
  *
  * Red Hat Linux release 6.2 (Zoot)
  * Red Hat Linux Advanced Server release 2.1AS (Pensacola)
+ * Red Hat Enterprise Linux AS release 3 (Taroon)
+ * Red Hat Enterprise Linux WS release 3 (Taroon)
  * Linux Mandrake release 7.1 (helium)
+ * Red Hat Enterprise Linux ES release 2.1 (Panama)
+ * Fedora Core release 1 (Yarrow)
  */
 
 #define RH_REL_FILENAME "/etc/redhat-release"
 
 FILE *fp;
 
-/* The full string red in from redhat-release */
-char relstring[maxvarsize];
-char classbuf[maxvarsize];
+/* The full string read in from redhat-release */
+char relstring[CF_MAXVARSIZE];
+char classbuf[CF_MAXVARSIZE];
 
 /* Red Hat, Mandrake */
 char *vendor="";
-/* as (Advanced Server) */
+/* as (Advanced Server, Enterprise) */
 char *edition="";
 /* Where the numerical release will be found */
 char *release=NULL;
 
 int major = -1;
-char strmajor[maxvarsize];
+char strmajor[CF_MAXVARSIZE];
 int minor = -1;
-char strminor[maxvarsize];
+char strminor[CF_MAXVARSIZE];
 
 /* Grab the first line from the file and then close it. */
  if ((fp = fopen(RH_REL_FILENAME,"r")) == NULL)
@@ -765,10 +1012,26 @@ char strminor[maxvarsize];
  fgets(relstring, sizeof(relstring), fp);
  fclose(fp);
  
- Verbose("Looking for redhat linux info...\n");
+Verbose("Looking for redhat linux info in \"%s\"\n",relstring);
  
  /* First, try to grok the vendor and the edition (if any) */
- if(!strncmp(relstring, REDHAT_AS_ID, strlen(REDHAT_AS_ID)))
+ if(!strncmp(relstring, REDHAT_ES_ID, strlen(REDHAT_ES_ID)))
+    {
+    vendor = "redhat";
+    edition = "es";
+    }
+ else if(!strncmp(relstring, REDHAT_WS_ID, strlen(REDHAT_WS_ID)))
+    {
+    vendor = "redhat";
+    edition = "ws";
+    }
+ else if(!strncmp(relstring, REDHAT_WS_ID, strlen(REDHAT_WS_ID)))
+    {
+    vendor = "redhat";
+    edition = "ws";
+    }
+ else if(!strncmp(relstring, REDHAT_AS_ID, strlen(REDHAT_AS_ID)) ||
+  !strncmp(relstring, REDHAT_AS21_ID, strlen(REDHAT_AS21_ID)))
     {
     vendor = "redhat";
     edition = "as";
@@ -781,6 +1044,10 @@ char strminor[maxvarsize];
     {
     vendor = "mandrake";
     }
+ else if(!strncmp(relstring, FEDORA_ID, strlen(FEDORA_ID)))
+    {
+    vendor = "fedora";
+    }
  else
     {
     Verbose("Could not identify OS distro from %s\n", RH_REL_FILENAME);
@@ -796,18 +1063,28 @@ char strminor[maxvarsize];
  if(release == NULL)
     {
     Verbose("Could not find a numeric OS release in %s\n",
-	    RH_REL_FILENAME);
+     RH_REL_FILENAME);
     return 2;
     }
  else
     {
     release += strlen(RELEASE_FLAG);
-    sscanf(release, "%d.%d", &major, &minor);
-    sprintf(strmajor, "%d", major);
-    sprintf(strminor, "%d", minor);
+    if (sscanf(release, "%d.%d", &major, &minor) == 2)
+       {
+       sprintf(strmajor, "%d", major);
+       sprintf(strminor, "%d", minor);
+       }
+    /* red hat 9 is *not* red hat 9.0. 
+     * and same thing with RHEL AS 3
+     */
+    else if (sscanf(release, "%d", &major) == 1)
+       {
+       sprintf(strmajor, "%d", major);
+       minor = -2;
+       };
     }
  
- if(major != -1 && minor != -1 && vendor != "")
+ if (major != -1 && minor != -1 && vendor != "")
     {
     classbuf[0] = '\0';
     strcat(classbuf, vendor);
@@ -821,11 +1098,100 @@ char strminor[maxvarsize];
        }
     strcat(classbuf, strmajor);
     AddClassToHeap(classbuf);
+    if (minor != -2)
+       {
+       strcat(classbuf, "_");
+       strcat(classbuf, strminor);
+       AddClassToHeap(classbuf);
+       }
+    }
+ return 0;
+}
+
+/******************************************************************/
+
+int linux_suse_version(void)
+{
+#define SUSE_REL_FILENAME "/etc/SuSE-release"
+#define SUSE_ID "SuSE Linux"
+#define SUSE_SLES8_ID "SuSE SLES-8"
+#define SUSE_SLES9_ID "SUSE LINUX Enterprise Server 9"
+#define SUSE_RELEASE_FLAG "Linux "
+
+/* The full string read in from SuSE-release */
+char relstring[CF_MAXVARSIZE];
+char classbuf[CF_MAXVARSIZE];
+
+/* Where the numerical release will be found */
+char *release=NULL;
+
+int major = -1;
+char strmajor[CF_MAXVARSIZE];
+int minor = -1;
+char strminor[CF_MAXVARSIZE];
+
+FILE *fp;
+
+ /* Grab the first line from the file and then close it. */
+ if ((fp = fopen(SUSE_REL_FILENAME,"r")) == NULL)
+    {
+    return 1;
+    }
+ fgets(relstring, sizeof(relstring), fp);
+ fclose(fp);
+
+  
+   /* Check if it's a SuSE Enterprise version
+   */
+ Verbose("Looking for SuSE enterprise info in \"%s\"\n",relstring);
+ 
+ if(!strncmp(relstring, SUSE_SLES8_ID, strlen(SUSE_SLES8_ID)))
+    {
+    classbuf[0] = '\0';
+    strcat(classbuf, "SLES8");
+    AddClassToHeap(classbuf);
+     }
+ else if(!strncmp(relstring, SUSE_SLES9_ID, strlen(SUSE_SLES9_ID)))
+    {
+    classbuf[0] = '\0';
+    strcat(classbuf, "SLES9");
+    AddClassToHeap(classbuf);
+    }
+ 
+   /* Determine release version. We assume that the version follows
+    * the string "SuSE Linux".
+    */
+ 
+ /* Determine release version. We assume that the version follows
+  * the string "SuSE Linux".
+  */
+ release = strstr(relstring, SUSE_RELEASE_FLAG);
+ if(release == NULL)
+    {
+    Verbose("Could not find a numeric OS release in %s\n",SUSE_REL_FILENAME);
+    return 2;
+    }
+ else
+    {
+    release += strlen(SUSE_RELEASE_FLAG);
+    sscanf(release, "%d.%d", &major, &minor);
+    sprintf(strmajor, "%d", major);
+    sprintf(strminor, "%d", minor);
+    }
+    if(major != -1 && minor != -1)
+    {
+    classbuf[0] = '\0';
+    strcat(classbuf, "SuSE");
+    AddClassToHeap(classbuf);
+    strcat(classbuf, "_");
+    strcat(classbuf, strmajor);
+    AddClassToHeap(classbuf);
     strcat(classbuf, "_");
     strcat(classbuf, strminor);
     AddClassToHeap(classbuf);
     }
- return 0;
+ 
+return 0;
 }
 
 /******************************************************************/
@@ -835,7 +1201,7 @@ int debian_version(void) /* Andrew Stribblehill */
 #define DEBIAN_VERSION_FILENAME "/etc/debian_version"
 int major = -1; 
 int release = -1;
-char classname[maxvarsize] = "";
+char classname[CF_MAXVARSIZE] = "";
 FILE *fp;
 
 if ((fp = fopen(DEBIAN_VERSION_FILENAME,"r")) == NULL)
@@ -848,12 +1214,12 @@ switch (fscanf(fp, "%d.%d", &major, &release))
     {
     case 2:
         Verbose("This appears to be a Debian %u.%u system.", major, release);
-        snprintf(classname, maxvarsize, "debian_%u_%u", major, release);
+        snprintf(classname, CF_MAXVARSIZE, "debian_%u_%u", major, release);
         AddClassToHeap(classname);
         /* Fall-through */
     case 1:
         Verbose("This appears to be a Debian %u system.", major);
-        snprintf(classname, maxvarsize, "debian_%u", major);
+        snprintf(classname, CF_MAXVARSIZE, "debian_%u", major);
         AddClassToHeap(classname);
         break;
     case 0:
@@ -864,3 +1230,102 @@ switch (fscanf(fp, "%d.%d", &major, &release))
 fclose(fp);
 return 0;
 }
+
+/******************************************************************/
+
+int linux_mandrake_version(void)
+{
+
+/* We are looking for one of the following strings... */
+#define MANDRAKE_ID "Linux Mandrake"
+#define MANDRAKE_REV_ID "Mandrake Linux"
+
+#define RELEASE_FLAG "release "
+#define MANDRAKE_REL_FILENAME "/etc/mandrake-release"
+
+FILE *fp;
+
+/* The full string read in from mandrake-release */
+char relstring[CF_MAXVARSIZE];
+char classbuf[CF_MAXVARSIZE];
+
+/* I've never seen Mandrake-Move or the other 'editions', so
+   I'm not going to try and support them here.  Contributions welcome. */
+
+/* Where the numerical release will be found */
+char *release=NULL;
+char *vendor=NULL;
+int major = -1;
+char strmajor[CF_MAXVARSIZE];
+int minor = -1;
+char strminor[CF_MAXVARSIZE];
+
+/* Grab the first line from the file and then close it. */
+ if ((fp = fopen(MANDRAKE_REL_FILENAME,"r")) == NULL)
+    {
+    return 1;
+    }
+ fgets(relstring, sizeof(relstring), fp);
+ fclose(fp);
+
+ Verbose("Looking for Mandrake linux info in \"%s\"\n",relstring);
+
+  /* Older Mandrakes had the 'Mandrake Linux' string in reverse order */
+ if(!strncmp(relstring, MANDRAKE_ID, strlen(MANDRAKE_ID)))
+    {
+    vendor = "mandrake";
+    }
+ else if(!strncmp(relstring, MANDRAKE_REV_ID, strlen(MANDRAKE_REV_ID)))
+    {
+    vendor = "mandrake";
+    }
+ else
+    {
+    Verbose("Could not identify OS distro from %s\n", MANDRAKE_REL_FILENAME);
+    return 2;
+    }
+
+ /* Now, grok the release. We assume that all the strings will
+  * have the word 'release' before the numerical release.
+  */
+ release = strstr(relstring, RELEASE_FLAG);
+ if(release == NULL)
+    {
+    Verbose("Could not find a numeric OS release in %s\n",
+     MANDRAKE_REL_FILENAME);
+    return 2;
+    }
+ else
+    {
+    release += strlen(RELEASE_FLAG);
+    if (sscanf(release, "%d.%d", &major, &minor) == 2)
+       {
+       sprintf(strmajor, "%d", major);
+       sprintf(strminor, "%d", minor);
+       }
+    else
+       {
+       Verbose("Could not break down release version numbers in %s\n",
+       MANDRAKE_REL_FILENAME);
+       };
+    }
+
+ if (major != -1 && minor != -1 && vendor != "")
+    {
+    classbuf[0] = '\0';
+    strcat(classbuf, vendor);
+    AddClassToHeap(classbuf);
+    strcat(classbuf, "_");
+    strcat(classbuf, strmajor);
+    AddClassToHeap(classbuf);
+    if (minor != -2)
+       {
+       strcat(classbuf, "_");
+       strcat(classbuf, strminor);
+       AddClassToHeap(classbuf);
+       }
+    }
+ return 0;
+}
+
+/******************************************************************/
