@@ -39,35 +39,32 @@ FILE *PIPE;
 
 /*******************************************************************/
 
-int LoadProcessTable(procdata,psopts)
-
-struct Item **procdata;
-char *psopts;
+int LoadProcessTable(struct Item **procdata,char *psopts)
 
 { FILE *pp;
-  char pscomm[maxlinksize], imgbackup;
+  char pscomm[CF_MAXLINKSIZE], imgbackup;
 
-snprintf(pscomm,maxlinksize,"%s %s",VPSCOMM[VSYSTEMHARDCLASS],psopts);
+snprintf(pscomm,CF_MAXLINKSIZE,"%s %s",VPSCOMM[VSYSTEMHARDCLASS],psopts);
 
 Verbose("%s: Running process command %s\n",VPREFIX,pscomm); 
   
 if ((pp = cfpopen(pscomm,"r")) == NULL)
    {
-   snprintf(OUTPUT,bufsize*2,"Couldn't open the process list with command %s\n",pscomm);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't open the process list with command %s\n",pscomm);
    CfLog(cferror,OUTPUT,"popen");
    return false;
    }
 
 while (!feof(pp))
    {
-   bzero(VBUFF,bufsize);
-   ReadLine(VBUFF,bufsize,pp);
+   memset(VBUFF,0,CF_BUFSIZE);
+   ReadLine(VBUFF,CF_BUFSIZE,pp);
    AppendItem(procdata,VBUFF,"");
    }
 
 cfpclose(pp);
 
-snprintf(VBUFF,maxvarsize,"%s/cf_procs",WORKDIR);
+snprintf(VBUFF,CF_MAXVARSIZE,"%s/state/cf_procs",WORKDIR);
 
 imgbackup = IMAGEBACKUP;
 IMAGEBACKUP = 'n'; 
@@ -78,12 +75,9 @@ return true;
 
 /*******************************************************************/
 
-void DoProcessCheck(pp,procdata)
+void DoProcessCheck(struct Process *pp,struct Item *procdata)
 
-struct Process *pp;
-struct Item *procdata;
-
-{ char line[bufsize];
+{ char line[CF_BUFSIZE];
   int matches=0,dosignals=true;
   mode_t maskval;
   struct stat statbuf;
@@ -112,46 +106,46 @@ if (pp->matches >= 0)
    switch (pp->comp)
       {
       case '=': if (matches != (int)pp->matches)
-                   {
-	    	   snprintf(OUTPUT,bufsize*2,"%d processes matched %s (should be %d)\n",matches,pp->expr,pp->matches);
-		   CfLog(cferror,OUTPUT,"");
-		   if (pp->action == 'm')
-		      {
-		      dosignals = true;
-		      }
-		   }
-                break;
-		
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"%d processes matched %s (should be %d)\n",matches,pp->expr,pp->matches);
+         CfLog(cferror,OUTPUT,"");
+         if (pp->action == 'm')
+            {
+            dosignals = true;
+            }
+         }
+          break;
+          
       case '>': if (matches < (int)pp->matches)
-	           {
-	    	   snprintf(OUTPUT,bufsize*2,"%d processes matched %s (should be >=%d)\n",matches,pp->expr,pp->matches);
-		   CfLog(cferror,OUTPUT,"");
-		   if (pp->action == 'm')
-		      {
-		      dosignals = true;
-		      }
-	           } 
-                break;
-		
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"%d processes matched %s (should be >=%d)\n",matches,pp->expr,pp->matches);
+         CfLog(cferror,OUTPUT,"");
+         if (pp->action == 'm')
+            {
+            dosignals = true;
+            }
+         } 
+          break;
+          
       case '<': if (matches > (int)pp->matches)
-	           {
-	    	   snprintf(OUTPUT,bufsize*2,"%d processes matched %s (should be <=%d)\n",matches,pp->expr,pp->matches);
-		   CfLog(cferror,OUTPUT,"");
-		   if (pp->action == 'm')
-		      {
-		      dosignals = true;
-		      }
-	           } 
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"%d processes matched %s (should be <=%d)\n",matches,pp->expr,pp->matches);
+         CfLog(cferror,OUTPUT,"");
+         if (pp->action == 'm')
+            {
+            dosignals = true;
+            }
+         } 
       }
    }
-
-if (dosignals)
-   {
-   DoSignals(pp,killlist);
-   }
  
-DeleteItemList(killlist);
-
+ if (dosignals)
+    {
+    DoSignals(pp,killlist);
+    }
+ 
+ DeleteItemList(killlist);
+ 
 if ((pp->action == 'm') && !dosignals && (matches != 0))
    {
    Verbose("%s: Matches found for %s - no restart sequence\n",VPREFIX,pp->expr);
@@ -161,6 +155,8 @@ if ((pp->action == 'm') && !dosignals && (matches != 0))
 if (strlen(pp->restart) != 0)
    {
    char argz[256];
+
+   Verbose("Existing restart sequence found (%s)\n",pp->restart);
    
    if ((matches != 0) && (pp->signal != cfkill) && (pp->signal != cfterm))
       {
@@ -172,12 +168,12 @@ if (strlen(pp->restart) != 0)
 
    if ((stat(argz,&statbuf) != -1) && (statbuf.st_mode & 0111 == 0))
       {
-      snprintf(OUTPUT,bufsize,"Restart sequence %s could not be executed (mode=%o), while searching (%s)",pp->restart,statbuf.st_mode &7777,pp->expr);
+      snprintf(OUTPUT,CF_BUFSIZE,"Restart sequence %s could not be executed (mode=%o), while searching (%s)",pp->restart,statbuf.st_mode &7777,pp->expr);
       CfLog(cferror,OUTPUT,"");
       return;
       }
    
-   snprintf(OUTPUT,bufsize*2,"Executing shell command: %s\n",pp->restart);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Executing shell command: %s\n",pp->restart);
    CfLog(cfinform,OUTPUT,"");
 
    if (DONTDO)
@@ -190,62 +186,62 @@ if (strlen(pp->restart) != 0)
    
    if (pp->umask == 0)
       {
-      snprintf(OUTPUT,bufsize*2,"Programming %s running with umask 0! Use umask= to set\n",pp->restart);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Programming %s running with umask 0! Use umask= to set\n",pp->restart);
       CfLog(cfsilent,OUTPUT,"");
       }
    
    if (pp->useshell == 'y')
       {
       if ((PIPE = cfpopen_shsetuid(pp->restart,"r",pp->uid,pp->gid,pp->chdir,pp->chroot)) == NULL)
-	 {
-	 snprintf(OUTPUT,bufsize*2,"Process restart execution failed on %s\n",pp->restart);
-	 CfLog(cferror,OUTPUT,"popen");
-	 return;
-	 }
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"Process restart execution failed on %s\n",pp->restart);
+         CfLog(cferror,OUTPUT,"popen");
+         return;
+         }
       }
    else
       {
       if ((PIPE = cfpopensetuid(pp->restart,"r",pp->uid,pp->gid,pp->chdir,pp->chroot)) == NULL)
-	 {
-	 snprintf(OUTPUT,bufsize*2,"Process restart execution failed on %s\n",pp->restart);
-	 CfLog(cferror,OUTPUT,"popen");
-	 return;
-	 }
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"Process restart execution failed on %s\n",pp->restart);
+         CfLog(cferror,OUTPUT,"popen");
+         return;
+         }
       }
-
+   
    DEADLOCK = false;   
    
    while (!feof(PIPE))
       {
       if (pp->useshell == 'd') /* dumb shell */
-	 {
-	 fgets(line,1,PIPE);
+         {
+         fgets(line,1,PIPE);
          break;
-	 }
+         }
+      
+      ReadLine(line,CF_BUFSIZE,PIPE);
       
-      ReadLine(line,bufsize,PIPE);
-
       if (feof(PIPE) || ferror(PIPE))
-	 {
-	 break;
-	 }
+         {
+         break;
+         }
       
       if (strstr(line,"cfengine-die"))
-	 {
-	 break;
-	 }
-
-      /* patch for ERESTARTSYSTEM bug in popen */
-
-      if (strstr(line,"cfd: start") || DEADLOCK)
-	 {
-	 break;
-	 }
+         {
+         break;
+         }
       
-      snprintf(OUTPUT,bufsize*2,"Restart: %s",line);
+      /* patch for ERESTARTSYSTEM bug in popen */
+      
+      if (strstr(line,"cfd: start") || DEADLOCK)
+         {
+         break;
+         }
+      
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Restart: %s",line);
       CfLog(cfinform,OUTPUT,"");
       }
-
+   
    if (pp->useshell == 'y')
       {
       pclose(PIPE);
@@ -255,7 +251,7 @@ if (strlen(pp->restart) != 0)
       cfpclose(PIPE);
       }
    
-   snprintf(OUTPUT,bufsize*2,"(Done with %s)\n",pp->restart);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"(Done with %s)\n",pp->restart);
    CfLog(cfinform,OUTPUT,"");
    umask(maskval);
    }
@@ -263,10 +259,7 @@ if (strlen(pp->restart) != 0)
 
 /**********************************************************************************/
 
-int FindMatches(pp,procdata,killlist)
-
-struct Process *pp;
-struct Item *procdata,**killlist;
+int FindMatches(struct Process *pp,struct Item *procdata,struct Item **killlist)
 
 { struct Item *ip, *ip2;
   char *sp,saveuid[16];
@@ -274,9 +267,9 @@ struct Item *procdata,**killlist;
   regex_t rx,rxcache;
   regmatch_t pmatch;
   pid_t cfengine_pid = getpid();
-  char *names[noproccols];      /* ps headers */
-  int start[noproccols];
-  int end[noproccols];
+  char *names[CF_PROCCOLS];      /* ps headers */
+  int start[CF_PROCCOLS];
+  int end[CF_PROCCOLS];
   
 Debug2("Looking for process %s\n",pp->expr);
 
@@ -289,7 +282,7 @@ GetProcessColumns(procdata->name,(char **)names,start,end);
 
 for (ip = procdata; ip != NULL; ip=ip->next)
    {
-   bcopy(&rxcache,&rx,sizeof(rx)); /* To fix a bug on some implementations where rx gets emptied */
+   memcpy(&rx,&rxcache,sizeof(rx)); /* To fix a bug on some implementations where rx gets emptied */
 
    if (regexec(&rx,ip->name,1,&pmatch,0) == 0)
       {
@@ -298,143 +291,143 @@ for (ip = procdata; ip != NULL; ip=ip->next)
       Debug("Regex %s matched %s\n",ip->name,pp->expr);
       
       for (ip2 = pp->inclusions; ip2 != NULL; ip2 = ip2->next)
-	 {
-	 got = false;
-	 
-	 if (strstr(ip->name,ip2->name) || WildMatch(ip2->name,ip->name))
-	    {
-	    got = true;
-	    break;
-	    }
-	 }
+         {
+         got = false;
+         
+         if (strstr(ip->name,ip2->name) || WildMatch(ip2->name,ip->name))
+            {
+            got = true;
+            break;
+            }
+         }
       
       if (!got)
-	 {
-	 continue;
-	 }
+         {
+         continue;
+         }
       
       got = false;
       
       for (ip2 = pp->exclusions; ip2 != NULL; ip2 = ip2->next)
-	 {
-	 if (strstr(ip->name,ip2->name) || WildMatch(ip2->name,ip->name))
-	    {
-	    got = true;
-	    break;
-	    }
-	 }
-
+         {
+         if (strstr(ip->name,ip2->name) || WildMatch(ip2->name,ip->name))
+            {
+            got = true;
+            break;
+            }
+         }
+      
       if (!ProcessFilter(ip->name,pp->filters,names,start,end))
-	 {
-	 Debug("%s Filtered away\n",ip->name);
-	 continue;
-	 }
+         {
+         Debug("%s Filtered away\n",ip->name);
+         continue;
+         }
       
       if (got)
-	 {
-	 continue;
-	 }
-
+         {
+         continue;
+         }
+      
       Debug("Matched proc[%s]\n",ip->name);
       
       for (sp = ip->name; *sp != '\0'; sp++) /* if first field contains alpha, skip */
-	 {
-	 while (true)
-	    {
-	    while (!isdigit((int)*sp) && (*sp != '\0'))
-	       {
-	       sp++;
-	       }
-	    
-	    if ((sp > ip->name) && isalnum((int)*(sp-1))) /* Username contains number*/
-	       {
-	       sp++;
-	       }
-	    else
-	       {
-	       break;
-	       }
-	    }
-	 
-	 sscanf(sp,"%d",&pid);
-	 
-	 if (pid != -1)
-	    {
-	    break;
-	    }
-	 }  
+         {
+         while (true)
+            {
+            while (!isdigit((int)*sp) && (*sp != '\0'))
+               {
+               sp++;
+               }
+            
+            if ((sp > ip->name) && isalnum((int)*(sp-1))) /* Username contains number*/
+               {
+               sp++;
+               }
+            else
+               {
+               break;
+               }
+            }
+  
+         sscanf(sp,"%d",&pid);
+         
+         if (pid != -1)
+            {
+            break;
+            }
+         }  
       
       if (pid == -1)
-	 {
-	 snprintf(OUTPUT,bufsize*2,"Unable to extract pid while looking for %s\n",pp->expr);
-	 CfLog(cfverbose,OUTPUT,"");
-	 continue;
-	 }
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"Unable to extract pid while looking for %s\n",pp->expr);
+         CfLog(cfverbose,OUTPUT,"");
+         continue;
+         }
       
       Debug2("Found matching pid %d\n",pid);
       
       matches++;
       
       if (pid == 1 && pp->signal == cfhup)
-	 {
-	 Verbose("(Okay to send HUP to init)\n");
-	 }
+         {
+         Verbose("(Okay to send HUP to init)\n");
+         }
       else if (pid < 4)
-	 {
-	 Verbose("%s: will not signal or restart processes 0,1,2,3\n",VPREFIX);
-	 Verbose("%s: occurred while looking for %s\n",VPREFIX,pp->expr);
-	 continue;
-	 }
+         {
+         Verbose("%s: will not signal or restart processes 0,1,2,3\n",VPREFIX);
+         Verbose("%s: occurred while looking for %s\n",VPREFIX,pp->expr);
+         continue;
+         }
       
       if (pp->action == 'w')
-	 {
-	 snprintf(OUTPUT,bufsize*2,"Process alert: %s\n",procdata->name);
-	 CfLog(cferror,OUTPUT,"");	 
-	 snprintf(OUTPUT,bufsize*2,"Process alert: %s\n",ip->name);
-	 CfLog(cferror,OUTPUT,"");
-	 continue;
-	 }
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"Process alert: %s\n",procdata->name);
+         CfLog(cferror,OUTPUT,"");  
+         snprintf(OUTPUT,CF_BUFSIZE*2,"Process alert: %s\n",ip->name);
+         CfLog(cferror,OUTPUT,"");
+         continue;
+         }
       
       if (pp->signal != cfnosignal)
-	 {
-	 if (!DONTDO)
-	    {
-	    if (pid == cfengine_pid)
-	       {
-	       CfLog(cfverbose,"Cfengine will not kill itself!\n","");
-	       continue;
-	       }
-
-	    if (pp->action == 'm')
-	       {
-	       sprintf(saveuid,"%d",pid);
-	       PrependItem(killlist,saveuid,"");
-	       }
-	    else
-	       {
-	       if ((ret = kill((pid_t)pid,pp->signal)) < 0)
-		  {
-		  snprintf(OUTPUT,bufsize*2,"Couldn't send signal to pid %d\n",pid);
-		  CfLog(cfverbose,OUTPUT,"kill");
-		  
-		  continue;
-		  }
-	       
-	       snprintf(OUTPUT,bufsize*2,"Signalled process %d (%s) with %s\n",pid,pp->expr,SIGNALS[pp->signal]);
-	       CfLog(cfinform,OUTPUT,"");
-	 
-	       if ((pp->signal == cfkill || pp->signal == cfterm) && ret >= 0)
-		  {
-		  snprintf(OUTPUT,bufsize*2,"Killed: %s\n",ip->name);
-		  CfLog(cfinform,OUTPUT,"");
-		  }
-	       }
-	    }
-	 }
+         {
+         if (!DONTDO)
+            {
+            if (pid == cfengine_pid)
+               {
+               CfLog(cfverbose,"Cfengine will not kill itself!\n","");
+               continue;
+               }
+            
+            if (pp->action == 'm')
+               {
+               sprintf(saveuid,"%d",pid);
+               PrependItem(killlist,saveuid,"");
+               }
+            else
+               {
+               if ((ret = kill((pid_t)pid,pp->signal)) < 0)
+                  {
+                  snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't send signal to pid %d\n",pid);
+                  CfLog(cfverbose,OUTPUT,"kill");
+                  
+                  continue;
+                  }
+               
+               snprintf(OUTPUT,CF_BUFSIZE*2,"Signalled process %d (%s) with %s\n",pid,pp->expr,SIGNALS[pp->signal]);
+               CfLog(cfinform,OUTPUT,"");
+               
+               if ((pp->signal == cfkill || pp->signal == cfterm) && ret >= 0)
+                  {
+                  snprintf(OUTPUT,CF_BUFSIZE*2,"Killed: %s\n",ip->name);
+                  CfLog(cfinform,OUTPUT,"");
+                  }
+               }
+            }
+         }
       }
    }
-
- for (i = 0; i < noproccols; i++)
+ 
+ for (i = 0; i < CF_PROCCOLS; i++)
     {
     if (names[i] != NULL)
        {
@@ -448,10 +441,7 @@ for (ip = procdata; ip != NULL; ip=ip->next)
 
 /**********************************************************************************/
 
-void DoSignals(pp,list)
-
-struct Process *pp;
-struct Item *list;
+void DoSignals(struct Process *pp,struct Item *list)
 
 { struct Item *ip;
   pid_t pid;
@@ -466,7 +456,8 @@ if (list == NULL)
 
 if (pp->signal == cfnosignal)
    {
-   Verbose ("No signal to send for %s\n",pp->expr);
+   snprintf(OUTPUT,CF_BUFSIZE,"No signal to send for %s\n",pp->expr);
+   CfLog(cfinform,OUTPUT,"");
    return;
    }
 
@@ -485,18 +476,18 @@ for (ip = list; ip != NULL; ip=ip->next)
    if (!DONTDO)
       {
       if ((ret = kill((pid_t)pid,pp->signal)) < 0)
-	 {
-	 snprintf(OUTPUT,bufsize*2,"Couldn't send signal to pid %d\n",pid);
-	 CfLog(cfverbose,OUTPUT,"kill");
-	 
-	 return;
-	 }
-   
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't send signal to pid %d\n",pid);
+         CfLog(cfverbose,OUTPUT,"kill");
+         
+         return;
+         }
+      
       if ((pp->signal == cfkill || pp->signal == cfterm) && ret >= 0)
-	 {
-	 snprintf(OUTPUT,bufsize*2,"Killed: %s\n",ip->name);
-	 CfLog(cfinform,OUTPUT,"");
-	 }
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"Killed: %s\n",ip->name);
+         CfLog(cfinform,OUTPUT,"");
+         }
       }
    }
 }
