@@ -36,15 +36,13 @@
 
 /*********************************************************************/
 
-int Repository(file,repository)
-
-char *file, *repository;
+int Repository(char *file,char *repository)
 
  /* Returns true if the file was backup up and false if not */
 
-{ char buffer[bufsize];
-  char localrepository[bufsize]; 
-  char node[maxlinksize];
+{ char buffer[CF_BUFSIZE];
+  char localrepository[CF_BUFSIZE]; 
+  char node[CF_BUFSIZE];
   struct stat sstat, dstat;
   char *sp;
   struct Image dummy;
@@ -52,7 +50,7 @@ char *file, *repository;
 
 if (repository == NULL)
    {
-   strcpy(localrepository,VREPOSITORY);
+   strncpy(localrepository,VREPOSITORY,CF_BUFSIZE);
    }
 else
    {
@@ -60,7 +58,8 @@ else
       {
       return false;
       }
-   strcpy(localrepository,repository);
+   
+   strncpy(localrepository,repository,CF_BUFSIZE);
    }
 
 if (IMAGEBACKUP == 'n')
@@ -70,9 +69,9 @@ if (IMAGEBACKUP == 'n')
 
 if (IsItemIn(VREPOSLIST,file))
    {
-   snprintf(OUTPUT,bufsize,"The file %s has already been moved to the repository once.",file);
+   snprintf(OUTPUT,CF_BUFSIZE,"The file %s has already been moved to the repository once.",file);
    CfLog(cfinform,OUTPUT,"");
-   snprintf(OUTPUT,bufsize,"Multiple update will cause loss of backup. Use backup=false in copy to override.");
+   snprintf(OUTPUT,CF_BUFSIZE,"Multiple update will cause loss of backup. Use backup=false in copy to override.");
    CfLog(cfinform,OUTPUT,"");
    return true;
    }
@@ -98,7 +97,7 @@ for (sp = node; *sp != '\0'; sp++)
       }
    }
 
-strcpy(buffer,localrepository);
+strncpy(buffer,localrepository,CF_BUFSIZE-2);
 AddSlash(buffer);
 
 if (BufferOverflow(buffer,node))
@@ -109,7 +108,10 @@ if (BufferOverflow(buffer,node))
 
 strcat(buffer,node);
 
-MakeDirectoriesFor(buffer,'y');
+if (!MakeDirectoriesFor(buffer,'y'))
+   {
+   snprintf(OUTPUT,CF_BUFSIZE,"Repository (%s),testfile (%s)",localrepository,buffer);
+   }
 
 if (stat(file,&sstat) == -1)
    {
@@ -120,7 +122,7 @@ if (stat(file,&sstat) == -1)
 stat(buffer,&dstat);
 
 imagecopy = IMAGEBACKUP;   /* without this there would be loop between this */
-IMAGEBACKUP = false;       /* and Repository */
+IMAGEBACKUP = 'n';       /* and Repository */
 
 dummy.server = "localhost";
 dummy.inode_cache = NULL;
@@ -128,6 +130,7 @@ dummy.cache = NULL;
 dummy.stealth = 'n';
 dummy.encrypt = 'n'; 
 dummy.preservetimes = 'n';
+dummy.repository = NULL; 
  
 CheckForHoles(&sstat,&dummy);
 
