@@ -34,11 +34,10 @@
 
 /*****************************************************************************/
 
-void InstallStrategy(alias,classes)
-
-char *alias, *classes;
+void InstallStrategy(char *alias,char *classes)
 
 { struct Strategy *ptr;
+  char ebuff[CF_EXPANDSIZE];
  
  Debug1("InstallStrategy(%s,%s)\n",alias,classes);
  
@@ -48,21 +47,21 @@ char *alias, *classes;
     return;
     }
  
- ExpandVarstring(alias,VBUFF,"");
+ ExpandVarstring(alias,ebuff,"");
  
  if ((ptr = (struct Strategy *)malloc(sizeof(struct Strategy))) == NULL)
     {
     FatalError("Memory Allocation failed for InstallStrategy() #1");
     }
  
- if ((ptr->name = strdup(VBUFF)) == NULL)
+ if ((ptr->name = strdup(ebuff)) == NULL)
     {
     FatalError("Memory Allocation failed in InstallStrategy");
     }
 
- ExpandVarstring(classes,VBUFF,"");
+ ExpandVarstring(classes,ebuff,"");
 
- if ((ptr->classes = strdup(VBUFF)) == NULL)
+ if ((ptr->classes = strdup(ebuff)) == NULL)
     {
     FatalError("Memory Allocation failed in InstallStrategy");
     }
@@ -78,6 +77,7 @@ char *alias, *classes;
  
  ptr->next = NULL;
  ptr->type = 'r';
+ ptr->done = 'n';
  ptr->strategies = NULL;
 
  VSTRATEGYLISTTOP = ptr;
@@ -85,12 +85,10 @@ char *alias, *classes;
 
 /*****************************************************************************/
 
-void AddClassToStrategy(alias,class,value)
-
-char *alias,*class,*value;
+void AddClassToStrategy(char *alias,char *class,char *value)
 
 { struct Strategy *sp;
-  char buf[maxvarsize];
+  char buf[CF_MAXVARSIZE],ebuff[CF_EXPANDSIZE];
   int val = -1;
 
 if (class[strlen(class)-1] != ':')
@@ -99,13 +97,13 @@ if (class[strlen(class)-1] != ':')
    return;
    }
   
-bzero(buf,maxvarsize);
+memset(buf,0,CF_MAXVARSIZE);
 sscanf(class,"%[^:]",&buf);
  
-ExpandVarstring(value,VBUFF,"");
-Debug("AddClassToStrategy(%s,%s,%s)\n",alias,class,VBUFF);
+ExpandVarstring(value,ebuff,"");
+Debug("AddClassToStrategy(%s,%s,%s)\n",alias,class,ebuff);
 
-sscanf(VBUFF,"%d",&val);
+sscanf(ebuff,"%d",&val);
 
 if (val <= 0)
    {
@@ -117,7 +115,7 @@ for (sp = VSTRATEGYLIST; sp != NULL; sp=sp->next)
    {
    if (strcmp(alias,sp->name) == 0)
       {
-      AppendItem(&(sp->strategies),buf,VBUFF);
+      AppendItem(&(sp->strategies),buf,ebuff);
       AddInstallable(buf);
       }
    }
@@ -131,51 +129,63 @@ void SetStrategies()
   struct Item *ip; 
   int total,count;
   double *array,*cumulative,cum,fluct;
- 
+
+  
 for (ptr = VSTRATEGYLIST; ptr != NULL; ptr=ptr->next)
    {
-   Verbose("Evaluating stratgey %s (type=%c)\n",ptr->name,ptr->type);
+   if (ptr->done == 'y')
+      {
+      continue;
+      }
+   else
+      {
+      ptr->done = 'y';
+      }
+   
+   Verbose("\n  Evaluating strategy %s (type=%c)\n",ptr->name,ptr->type);
    if (ptr->strategies)
       {
       total = count = 0;
       for (ip = ptr->strategies; ip !=NULL; ip=ip->next)
-	 {
-	 count++;
-	 total += atoi(ip->classes);
-	 }
-
+         {
+         count++;
+         total += atoi(ip->classes);
+         }
+      
       count++;
       array = (double *)malloc(count*sizeof(double));
       cumulative = (double *)malloc(count*sizeof(double));
       count = 1;
       cum = 0.0;
       cumulative[0] = 0;
-            
+      
       for (ip = ptr->strategies; ip !=NULL; ip=ip->next)
-	 {
-	 array[count] = ((double)atoi(ip->classes))/((double)total);
-	 cum += array[count];
-	 cumulative[count] = cum;
-	 Debug("%s : %f cum %f\n",ip->name,array[count],cumulative[count]);
-	 count++;
-	 }
-
+         {
+         array[count] = ((double)atoi(ip->classes))/((double)total);
+         cum += array[count];
+         cumulative[count] = cum;
+         Debug("%s : %f cum %f\n",ip->name,array[count],cumulative[count]);
+         count++;
+         }
+      
       /* Get random number 0-1 */
-
+      
       fluct = drand48();
-
+      
       count = 1;
       cum = 0.0;
-
+      
       for (ip = ptr->strategies; ip !=NULL; ip=ip->next)
-	 {
-	 if ((cumulative[count-1] < fluct) && (fluct < cumulative[count]))
-	    {
-	    Verbose(" - Choosing %s (%f)\n",ip->name,fluct);
-	    AddClassToHeap(ip->name);
-	    }
-	 count++;
-	 }
+         {
+         Verbose("    Class %d: %f-%f\n",count,cumulative[count-1],cumulative[count]);
+         if ((cumulative[count-1] < fluct) && (fluct < cumulative[count]))
+            {
+            Verbose("   - Choosing %s (%f)\n",ip->name,fluct);
+            AddClassToHeap(ip->name);
+            break;
+            }
+         count++;
+         }
       free(cumulative);
       free(array);
       }
