@@ -38,10 +38,7 @@
 
 /*******************************************************************/
 
-void TidyWrapper(startpath,vp)
-
-char *startpath;
-void *vp;
+void TidyWrapper(char *startpath,void *vp)
 
 { struct Tidy *tp;
   struct stat sb; 
@@ -57,18 +54,19 @@ if (tp->done == 'y')
 
 tp->done = 'y'; 
 
-if (!GetLock(ASUniqueName("tidy"),CanonifyName(startpath),VIFELAPSED,VEXPIREAFTER,VUQNAME,CFSTARTTIME))
+if (!GetLock(ASUniqueName("tidy"),CanonifyName(startpath),tp->ifelapsed,tp->expireafter,VUQNAME,CFSTARTTIME))
    {
    return;
    }
 
 if (stat(startpath,&sb) == -1)
    {
-   snprintf(OUTPUT,bufsize,"Tidy directory %s cannot be accessed",startpath);
+   snprintf(OUTPUT,CF_BUFSIZE,"Tidy directory %s cannot be accessed",startpath);
    CfLog(cfinform,OUTPUT,"stat");
    return;
    }
- 
+
+RegisterRecursionRootDevice(sb.st_dev);
 RecursiveTidySpecialArea(startpath,tp,tp->maxrecurse,&sb);
 
 ReleaseCurrentLock(); 
@@ -76,27 +74,36 @@ ReleaseCurrentLock();
 
 /*******************************************************************/
 
-void RecHomeTidyWrapper(startpath,vp)
-
-char *startpath;
-void *vp;
+void RecHomeTidyWrapper(char *startpath,void *vp)
 
 { struct stat sb;
+  struct Tidy *tp = (struct Tidy *) vp;
  
 Verbose("Tidying Home partition %s...\n",startpath);
 
-if (!GetLock(ASUniqueName("tidy"),CanonifyName(startpath),VIFELAPSED,VEXPIREAFTER,VUQNAME,CFSTARTTIME))
-   {
-   return;
-   }
+ if (tp != NULL)
+    {
+    if (!GetLock(ASUniqueName("tidy"),CanonifyName(startpath),tp->ifelapsed,tp->expireafter,VUQNAME,CFSTARTTIME))
+       {
+       return;
+       }
+    }
+ else
+    {
+    if (!GetLock(ASUniqueName("tidy"),CanonifyName(startpath),VIFELAPSED,VEXPIREAFTER,VUQNAME,CFSTARTTIME))
+       {
+       return;
+       }
+    }
 
 if (stat(startpath,&sb) == -1)
    {
-   snprintf(OUTPUT,bufsize,"Tidy directory %s cannot be accessed",startpath);
+   snprintf(OUTPUT,CF_BUFSIZE,"Tidy directory %s cannot be accessed",startpath);
    CfLog(cfinform,OUTPUT,"stat");
    return;
    }
 
+RegisterRecursionRootDevice(sb.st_dev); 
 RecursiveHomeTidy(startpath,1,&sb);
  
 ReleaseCurrentLock();
@@ -104,14 +111,11 @@ ReleaseCurrentLock();
 
 /*******************************************************************/
 
-void CheckFileWrapper(startpath,vp)
-
-char *startpath;
-void *vp;
+void CheckFileWrapper(char *startpath,void *vp)
 
 {  struct stat statbuf;
    mode_t filemode;
-   char *lastnode, lock[maxvarsize];
+   char *lastnode, lock[CF_MAXVARSIZE];
    int fd;
    struct File *ptr;
 
@@ -119,15 +123,15 @@ ptr = (struct File *)vp;
 
 if (ptr->uid != NULL)
    {
-   snprintf(lock,maxvarsize-1,"%s_%o_%o_%d",startpath,ptr->plus,ptr->minus,ptr->uid->uid);
+   snprintf(lock,CF_MAXVARSIZE-1,"%s_%o_%o_%d",startpath,ptr->plus,ptr->minus,ptr->uid->uid);
    }
 else
    {
-   snprintf(lock,maxvarsize-1,"%s_%o_%o",startpath,ptr->plus,ptr->minus);
+   snprintf(lock,CF_MAXVARSIZE-1,"%s_%o_%o",startpath,ptr->plus,ptr->minus);
    }
  
  
-if (!GetLock(ASUniqueName("files"),CanonifyName(lock),VIFELAPSED,VEXPIREAFTER,VUQNAME,CFSTARTTIME))
+if (!GetLock(ASUniqueName("files"),CanonifyName(lock),ptr->ifelapsed,ptr->expireafter,VUQNAME,CFSTARTTIME))
    {
    return;
    }
@@ -148,7 +152,7 @@ for (lastnode = startpath+strlen(startpath)-1; *lastnode != '/'; lastnode--)
    }
 
 lastnode++;
-
+ 
 if (ptr->inclusions != NULL && !IsWildItemIn(ptr->inclusions,lastnode))
    {
    Debug2("cfengine: skipping non-included pattern %s\n",lastnode);
@@ -156,9 +160,9 @@ if (ptr->inclusions != NULL && !IsWildItemIn(ptr->inclusions,lastnode))
    if (stat(startpath,&statbuf) != -1)
       {
       if (!S_ISDIR(statbuf.st_mode))
-	 {
-	 return;  /* assure that we recurse into directories */
-	 }
+  {
+  return;  /* assure that we recurse into directories */
+  }
       }
    }
 
@@ -171,7 +175,7 @@ if (IsWildItemIn(ptr->exclusions,lastnode))
     
 if (stat(startpath,&statbuf) == -1)
    {
-   snprintf(OUTPUT,bufsize*2,"Cannot access file/directory %s\n",ptr->path);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Cannot access file/directory %s\n",ptr->path);
    CfLog(cfinform,OUTPUT,"stat");
 
    if (TouchDirectory(ptr))                /* files ending in /. */
@@ -179,7 +183,7 @@ if (stat(startpath,&statbuf) == -1)
       MakeDirectoriesFor(startpath,'n');
       ptr->action = fixall;
       *(startpath+strlen(ptr->path)-2) = '\0';       /* trunc /. */
-      CheckExistingFile(startpath,ptr->plus,ptr->minus,ptr->action,ptr->uid,ptr->gid,&statbuf,ptr,ptr->acl_aliases);
+      CheckExistingFile("*",startpath,ptr->plus,ptr->minus,ptr->action,ptr->uid,ptr->gid,&statbuf,ptr,ptr->acl_aliases);
       ReleaseCurrentLock();
       return;
       }
@@ -191,28 +195,29 @@ if (stat(startpath,&statbuf) == -1)
    switch (ptr->action)
       {
       case create:
-      case touch:   if (! DONTDO)
-                       {
-                       MakeDirectoriesFor(startpath,'n');
-		       
-                       if ((fd = creat(ptr->path,filemode)) == -1)
-			  { 
-			  perror("creat");
-			  AddMultipleClasses(ptr->elsedef);
-			  return;
-			  }
-		       else
-			  {
-			  AddMultipleClasses(ptr->defines);
-			  close(fd);
-			  }
-
-		       CheckExistingFile(startpath,ptr->plus,ptr->minus,fixall,ptr->uid,ptr->gid,&statbuf,ptr,ptr->acl_aliases);
-                       }
-
-                    snprintf(OUTPUT,bufsize*2,"Creating file %s, mode = %o\n",ptr->path,filemode);
-		    CfLog(cfinform,OUTPUT,"");
-                    break;
+      case touch:
+          if (! DONTDO)
+             {
+             MakeDirectoriesFor(startpath,'n');
+             
+             if ((fd = creat(ptr->path,filemode)) == -1)
+                { 
+                perror("creat");
+                AddMultipleClasses(ptr->elsedef);
+                return;
+                }
+         else
+            {
+            AddMultipleClasses(ptr->defines);
+            close(fd);
+            }
+             
+             CheckExistingFile("*",startpath,ptr->plus,ptr->minus,fixall,ptr->uid,ptr->gid,&statbuf,ptr,ptr->acl_aliases);
+             }
+          
+          snprintf(OUTPUT,CF_BUFSIZE*2,"Creating file %s, mode = %o\n",ptr->path,filemode);
+          CfLog(cfinform,OUTPUT,"");
+          break;
       case alert:
       case linkchildren: 
       case warnall:
@@ -220,12 +225,13 @@ if (stat(startpath,&statbuf) == -1)
       case warndirs:
       case fixplain:
       case fixdirs:
-      case fixall:  snprintf(OUTPUT,bufsize*2,"File/Dir %s did not exist and was marked (%s)\n",ptr->path,FILEACTIONTEXT[ptr->action]);
-	            CfLog(cfinform,OUTPUT,"");
-                    break;
+      case fixall:
+          snprintf(OUTPUT,CF_BUFSIZE*2,"File/Dir %s did not exist and was marked (%s)\n",ptr->path,FILEACTIONTEXT[ptr->action]);
+          CfLog(cfinform,OUTPUT,"");
+          break;
       case compress:
-	            break;
-
+          break;
+          
       default:      FatalError("cfengine: Internal sofware error: Checkfiles(), bad action\n");
       }
    }
@@ -243,55 +249,59 @@ else
       ReleaseCurrentLock();
       return;
       }
-
+   
+   
    if (ptr->action == linkchildren)
       {
       LinkChildren(ptr->path,
-		   's',
-		   &statbuf,
-		   ptr->uid->uid,ptr->gid->gid,
-		   ptr->inclusions,
-		   ptr->exclusions,
-		   NULL,
-		   0,
-		   NULL);
+                   's',
+                   &statbuf,
+                   ptr->uid->uid,ptr->gid->gid,
+                   ptr->inclusions,
+                   ptr->exclusions,
+                   NULL,
+                   0,
+                   NULL);
       ReleaseCurrentLock();
       return;
       }
-
+   
    if (S_ISDIR(statbuf.st_mode) && (ptr->recurse != 0))
       {
-      if (!IgnoreFile(startpath,ReadLastNode(startpath),ptr->ignores))
-	 {
-	 Verbose("%s: Skipping ignored directory %s\n",VPREFIX,startpath);
-	 CheckExistingFile(startpath,ptr->plus,ptr->minus,ptr->action,ptr->uid,ptr->gid,&statbuf,ptr,ptr->acl_aliases);
-	 }
-          
+      if (IgnoreFile(startpath,ReadLastNode(startpath),ptr->ignores))
+         {
+         Verbose("%s: (Skipping ignored directory %s)\n",VPREFIX,startpath);
+         return;
+         }
+      
+      CheckExistingFile("*",startpath,ptr->plus,ptr->minus,ptr->action,ptr->uid,ptr->gid,&statbuf,ptr,ptr->acl_aliases);
+      RegisterRecursionRootDevice(statbuf.st_dev);
       RecursiveCheck(startpath,ptr->plus,ptr->minus,ptr->action,ptr->uid,ptr->gid,ptr->recurse,0,ptr,&statbuf);
       }
    else
       {
-      CheckExistingFile(startpath,ptr->plus,ptr->minus,ptr->action,ptr->uid,ptr->gid,&statbuf,ptr,ptr->acl_aliases);
+      if (lstat(startpath,&statbuf) == -1)
+         {
+         CfLog(cferror,"Unable to stat already statted object!","lstat");
+         return;
+         }
+      CheckExistingFile("*",startpath,ptr->plus,ptr->minus,ptr->action,ptr->uid,ptr->gid,&statbuf,ptr,ptr->acl_aliases);
       }
    }
-
+ 
 ReleaseCurrentLock();
 }
 
 /*******************************************************************/
 
-void DirectoriesWrapper(dir,vp)
-
-char *dir;
-void *vp;
+void DirectoriesWrapper(char *dir,void *vp)
 
 { struct stat statbuf;
-  char directory[bufsize];
+  char directory[CF_EXPANDSIZE];
   struct File *ptr;
 
 ptr=(struct File *)vp;
  
-bzero(directory,bufsize);
 ExpandVarstring(dir,directory,"");
 
 AddSlash(directory);
@@ -301,24 +311,24 @@ MakeDirectoriesFor(directory,'n');
 
 if (stat(directory,&statbuf) == -1)
    {
-   bzero(directory,bufsize);
+   memset(directory,0,CF_BUFSIZE);
    ExpandVarstring(dir,directory,"");
    chmod(directory,0500); /* Shouldn't happen - mode 000 ??*/
  
    if (stat(directory,&statbuf) == -1)
       {
-      snprintf(OUTPUT,bufsize*2,"Cannot stat %s after creating it",directory);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Cannot stat %s after creating it",directory);
       CfLog(cfinform,OUTPUT,"stat");
       return;
       }
    } 
 
-if (!GetLock(ASUniqueName("directories"),CanonifyName(directory),VIFELAPSED,VEXPIREAFTER,VUQNAME,CFSTARTTIME))
+if (!GetLock(ASUniqueName("directories"),CanonifyName(directory),ptr->ifelapsed,ptr->expireafter,VUQNAME,CFSTARTTIME))
    {
    return;
    }
 
-CheckExistingFile(dir,ptr->plus,ptr->minus,ptr->action,ptr->uid,ptr->gid,&statbuf,ptr,ptr->acl_aliases);
+CheckExistingFile("*",dir,ptr->plus,ptr->minus,ptr->action,ptr->uid,ptr->gid,&statbuf,ptr,ptr->acl_aliases);
 ReleaseCurrentLock(); 
 }
 
